package jbox.agent;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.instrument.ClassDefinition;
import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.instrument.Instrumentation;
import java.lang.instrument.UnmodifiableClassException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jbox.agent.instrumentation.JBoxClassFileTransformer;
import jbox.agent.instrumentation.JBoxObjectTransformer;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.helpers.Loader;

/**
 * -javaagent:
 */
public class Agent {
	

	private static final Logger LOG;
	private static boolean installed = false;
	
	public static boolean isAgentActive = false;
	
	private static ClassFileTransformer objecttransformer;
	private static ClassFileTransformer transformer;
	
	public static List<String> except;

	static {
		// use the loader helper from log4j
		
		URL url = Loader.getResource("agent.properties");
		PropertyConfigurator.configure(url);

		LOG = Logger.getLogger(Agent.class);
	}
	
	public static void premain(String agentArgs, Instrumentation instrumentation) {
		
		for (Class<?> klass : instrumentation.getAllLoadedClasses()) {
			//System.out.println(klass.getName());
		}
		
		if (installed) {

			LOG.warn("one JBox Agent was already installed.");
			LOG
					.warn("your probably have the -javaagent arg on the command line twice");

			return;
		}

		LOG.info("Installing JBox ClassFileTransformer...");

		try {

			objecttransformer = new JBoxObjectTransformer();
			transformer = new JBoxClassFileTransformer();
			
			redefineClasses(instrumentation);
			
			//LOG.info("succeed");

			instrumentation.addTransformer(transformer);
			installed = true;

			// LOG.info("Installed " + transformer + ".");
			
		} catch (Throwable e) {

			e.printStackTrace();
			LOG.info("Installed = " + installed, e);

		}
	}

	private static void redefineClasses(Instrumentation instrumentation) {
		if (!instrumentation.isRedefineClassesSupported()) {
			LOG.info("RedefineClassesSupported = false");
			//TODO: Abort program, no meaningful way to continue
			return;
		}
		
		BufferedReader excpt = new BufferedReader(new InputStreamReader(Agent.class.getResourceAsStream("notmodify")));
		except = new ArrayList<String>();
		String s;
		try {
			while ((s = excpt.readLine()) != null) {
				except.add(s);
			}
		} catch (IOException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}

		ArrayList<ClassDefinition> defs = new ArrayList<ClassDefinition>();
		for (Class<?> klass : instrumentation.getAllLoadedClasses()) {
			
			

			
			String resourceName = "/" + klass.getName().replaceAll("\\.", "/")
					+ ".class";
			InputStream inputStream = klass.getResourceAsStream(resourceName);

			if (inputStream == null && !klass.isArray()) {
				LOG.debug("Can not find class for transformation: " + klass.getName());
			}
			
			byte[] newBytes = null;
			
			byte[] bytes = null;

			if (!klass.getName().startsWith("jbox.Agent") && inputStream != null) {
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream(4096);
				
				byte[] buffer = new byte[4096];
		        int b = 0;

		        try {
		            for (b = 0; b >= 0; b = inputStream.read(buffer)) {
		                if (b != 0) {
		                	outputStream.write(buffer, 0, b);
		                }
		            }
		        } catch (IOException e) {
		        	LOG.fatal("could not read object class file");
		        }

				bytes = outputStream.toByteArray();

			    
				/*try {
					newBytes = objecttransformer.transform(klass.getClassLoader(),
							klass.getName(), klass,
							klass.getProtectionDomain(), bytes);
				} catch (IllegalClassFormatException e1) {
					e1.printStackTrace();
				}*/
				
				try {
					if (!except.contains(klass.getName())) {
						newBytes = transformer.transform(klass.getClassLoader(),
								klass.getName(), klass,
								klass.getProtectionDomain(), bytes);
					}
				} catch (IllegalClassFormatException e1) {
					e1.printStackTrace();
				}
				
				try {
					inputStream.close();
					outputStream.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}

			if (newBytes != null) {
				defs.add(new ClassDefinition(klass, newBytes));
			}
			
		}
		ClassDefinition[] cdef = defs.toArray(new ClassDefinition[defs.size()]);
		try {
			/*
			 * DEBUG
			 */
			/*for (int i = 0; i < defs.size(); i++) {
				System.out.println("try for " + defs.get(i).getDefinitionClass());
				instrumentation.redefineClasses(new ClassDefinition[] {defs.get(i)});
				System.out.println("ok  for " + defs.get(i).getDefinitionClass());
			}*/
			
			instrumentation.redefineClasses(cdef);
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnmodifiableClassException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Throwable t) {
			t.printStackTrace();
			
		}
	}
	


}
