package jbox.agent.instrumentation;


import org.apache.log4j.Logger;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;
import org.objectweb.asm.commons.GeneratorAdapter;
import org.objectweb.asm.commons.Method;

public class BoxTransformer extends ClassAdapter implements Opcodes {

	private static final org.apache.log4j.Logger LOG = Logger.getLogger(BoxTransformer.class);
	
    public BoxTransformer(ClassVisitor visitor) {
        super(visitor);
    }
    
    public MethodVisitor visitMethod(int access, String name, String descriptor, String signature,
            String[] exceptions) {
    	MethodVisitor visitor = super.visitMethod(access, name, descriptor, signature, exceptions);
    	return new BVisitor(visitor, access, name, descriptor);
    }
    
    public void visit(int version, int access, String name, String signature, String superName, String[] interfaces) {
    	super.visit(version, access, name, signature, superName, interfaces);
    }
    
   
   
    class BVisitor extends GeneratorAdapter {
    	String methodName;
    	String methodDesc;
    	boolean found = false;
    	String specHandler;
    	String spec;
    	
    	public BVisitor(MethodVisitor arg0, int access, String name, String desc) {
			super(arg0, access, name, desc);
			this.methodName = name;
			this.methodDesc = desc;
		}
    	
    	@Override
    	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
    		final AnnotationVisitor visitor = super.visitAnnotation(desc, visible);
    		if (desc.equals("Ljbox/runtime/annotation/newBox;")) {
    			LOG.debug("Box implementation found");
    			found = true;

    			
    			return new AnnotationVisitor() {

    				public void visit(String arg0, Object arg1) {
    					if (arg0.equals("spec")) {
    						spec = arg1.toString();
    					}
    					if (arg0.equals("specHandler")) {
    						Type clazz = (Type)arg1;
    						specHandler = clazz.getClassName();
    					}
    					visitor.visit(arg0, arg1);
    				}

    				public AnnotationVisitor visitAnnotation(String arg0, String arg1) {
    					return visitor.visitAnnotation(arg0, arg1);
    				}

    				public AnnotationVisitor visitArray(String arg0) {
    					return visitor.visitArray(arg0);
    				}

    				public void visitEnd() {
    					visitor.visitEnd();
    				}

    				public void visitEnum(String arg0, String arg1, String arg2) {
    					visitor.visitEnum(arg0, arg1, arg2);
    				}

    			};
    			
    		}

    		return visitor;
    	}
    	 
    	@Override
    	public void visitMethodInsn(int opcode, String owner, String name, String desc) {
    			//TODO: needs further restriction
    			if (this.methodName.equals("create") &&
    					name.equals("<init>") && found) {
    				
    				
    				this.push(Type.getReturnType(methodDesc).getClassName());
    				this.push(specHandler);
    				this.push(spec);
    				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
    						Method.getMethod("void nextObjectNewBox(String, String, String)"));
					
    				//super.visitMethodInsn(INVOKESTATIC, "jbox/runtime/Handler", "nextObjectNewBox", "()V");
    				
    				/*this.invokeStatic(Type.getType(Handler.class), new Method(
							"nextObjectNewBox", Type
									.getMethodDescriptor(Handler.class
											.getMethod("nextObjectNewBox"))));
					*/
    			}
				super.visitMethodInsn(opcode, owner, name, desc);
    	}
    	
    	
    }

}
