package jbox.agent.instrumentation;


import org.objectweb.asm.*;
import org.objectweb.asm.commons.AdviceAdapter;
import org.objectweb.asm.commons.Method;

import java.lang.reflect.Modifier;

import org.apache.log4j.Logger;


public class MethodTransformer extends ClassAdapter implements Opcodes {

	private static final org.apache.log4j.Logger LOG = Logger.getLogger(MethodTransformer.class);

	String className;



	public MethodTransformer(ClassVisitor visitor, String className) {
		super(visitor);
		this.className = className;
	}

	@Override
	public MethodVisitor visitMethod(int access, String name, String descriptor, String signature,
			String[] exceptions) {

		MethodVisitor visitor = super.visitMethod(access, name, descriptor, signature, exceptions);

		if (name.equals("finalize")) {
			return visitor;
		}   	
		if (name.equals("<clinit>")) {
			return visitor;
		}

		return new CVisitor(visitor, access, name, descriptor, className);
	}

	static class CVisitor extends AdviceAdapter {
		int access;
		String name;
		String className;

		public CVisitor(MethodVisitor arg0, int access, String name, String desc, String className) {
			super(arg0, access, name, desc);
			this.access = access;
			this.name = name;
			this.className = className;
		}



		@Override
		public void onMethodEnter() {

			if (className.equals("java/lang/Object") && name.equals("<init>")) {
				super.onMethodEnter();
				return;
			}
			if (Modifier.isStatic(access)) {
				/*this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onMethodEnter()"));*/
			} else {
				this.loadThis();
				this.loadArgArray();
				this.push(Type.getObjectType(className).getClassName());
				this.push(name);
				this.push(methodDesc);

				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onMethodEnter(Object, Object[], String, String, String)"));//new Method("onMethodEnter", "(Ljava/lang/Object;[Ljava/lang/Object;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V"));
			}

			super.onMethodEnter();
		}

		@Override
		public void onMethodExit(int opcode) {

			if (Modifier.isStatic(access)) {
				//this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), Method.getMethod("void onMethodExitS()"));
				
			} else {
				if (className.equals("java.lang.Object") && name.equals("<init>")) {
					this.loadThis();
					this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
							Method.getMethod("void onNewObject(Object)"));
				} else {
					//get return value
					if(opcode==RETURN) {
						visitInsn(ACONST_NULL);
					} else if(opcode==ARETURN || opcode==ATHROW) {
						dup();
					} else {
						if(opcode==LRETURN || opcode==DRETURN) {
							dup2();
						} else {
							dup();
						}
						box(Type.getReturnType(this.methodDesc));
					}

					this.loadThis();
					this.loadArgArray();

					this.push(Type.getObjectType(className).getClassName());
					this.push(name);
					this.push(methodDesc);
					this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
							Method.getMethod("void onMethodExit(Object, Object, Object[], String, String, String)"));
				}
			}

			super.onMethodExit(opcode);
		}

		/**
		 * According to vm spec for getfield:
		 * Operand Stack:
		 * ...,objectref => ...,value
		 * for putfield:
		 * Operand Stack:
		 * ...,objectref,value => ...
		 */
		@Override
		public void visitFieldInsn(int opcode, String owner, String name, String desc) {

			if (opcode == GETFIELD) {
				//this.getField(Type.getType(owner), name, Type.getType(desc));
				this.dup();
				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onFieldAccess(Object)"));	
			}

			if (opcode == PUTFIELD) {
				//this.getField(Type.getType(owner), name, Type.getType(desc));
				int local = newLocal(Type.getType(desc));
				storeLocal(local);
				this.dup();
				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onFieldAccess(Object)"));

				loadLocal(local);

			}

			super.visitFieldInsn(opcode, owner, name, desc);
		}

		/**
		 * checks outgoing calls
		 * invokespecial is always an internal call
		 * invokevirtual & invokespecial:
		 * Operand stack
		 * ..., objectref, [arg1, [arg2 ...]]  ...
		 */
		/*@Override
		public void visitMethodInsn(int opcode, String owner, String name, String desc) {
			if (className.equals("java/lang/Object")) {
				super.visitMethodInsn(opcode, owner, name, desc);
				return;
			}
			if (opcode == INVOKEVIRTUAL) {
				//TODO: Check why this doesn't work for Object
				//if (!owner.equals("java/lang/Object")) {
				//LOG.debug("Method " + owner + "." + name + "(" + desc + ")");
				Type[] args = Type.getArgumentTypes(desc);	
				int oRef = tempstore(args.length - 1, args, desc, owner, name);
					
				super.visitMethodInsn(opcode, owner, name, desc);
				
				this.loadLocal(oRef);
				this.push(Type.getObjectType(owner).getClassName());
				this.push(name);
				this.push(desc);
				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onMethodReturn(Object, String, String, String)"));
				
			} else {
				super.visitMethodInsn(opcode, owner, name, desc);
			}
		}*/

		/**
		 * Helper method to get the objectref from invokespecial & invokevirtual
		 * @param currindex
		 * @param types
		 * @param owner
		 * @param desc
		 * @throws NoSuchMethodException 
		 * @throws SecurityException 
		 */
		private int tempstore(int currindex, Type[] types,String desc, String owner, String name) {
			int result;
			if (currindex >= 0) {
				Type currtype = types[currindex];
				//LOG.debug("Storing " + currtype.getClassName());
				int local = newLocal(currtype);
				storeLocal(local);

				result = tempstore(currindex - 1, types, desc, owner, name);

				loadLocal(local);
			} else {
				
				this.dup();
				result = newLocal(Type.getObjectType(owner));
				storeLocal(result);
				this.dup();
				this.push(Type.getObjectType(owner).getClassName());
				this.push(name);
				this.push(desc);
				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onMethodCall(Object, String, String, String)"));
			}
			
			return result;

		}
		
		@Override
		public void visitTypeInsn(int opcode, String type) {
			if (opcode == CHECKCAST || opcode == INSTANCEOF) {
				this.dup();
				this.push(Type.getObjectType(type).getClassName());
				this.invokeStatic(Type.getObjectType("jbox/agent/Handler"), 
						Method.getMethod("void onCast(Object, String)"));
			}
			super.visitTypeInsn(opcode, type);
		}
	}


	//TODO : getStatic und putStatic berschreiben?
}