package jbox.runtime;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;

public class Boundary {
	private static final Logger LOG = Logger.getLogger(Boundary.class);
	private static HashMap<Class<?>, Boundary> boundary = new HashMap<Class<?>, Boundary>();
	
	private Set<Class<?>> classes;
	
	private Boundary() {}
	
	public static synchronized Boundary getBoundary(Class<?> clazz) {
		Boundary b = boundary.get(clazz);
		if (b == null) {
			b = new Boundary();
			result = new HashSet<Class<?>>();
			generateBoundary(clazz);
			b.classes = result;
			result = null;
			boundary.put(clazz, b);
			
			if (b.classes.size() > 100) {
				LOG.warn("Huge boundary for class " + clazz + " : " + b.classes.size());
			}
		}
		return b;
	}
	
	
	private static Set<Class<?>> result = null;
	
	private static void generateBoundary(Class<?> clazz) {
		Class<?> superclazz = clazz.getSuperclass();
		if (superclazz != null && !result.contains(superclazz)) {
			result.add(superclazz);
			generateBoundary(superclazz);
		}
		for (Class<?> c : clazz.getInterfaces()) {
			if (!result.contains(c)) {
				result.add(c);
				generateBoundary(c);
			}
		}
		for (Constructor<?> m : clazz.getDeclaredConstructors()) {
			for (Class<?> c : m.getParameterTypes()) {
				if (!result.contains(c)) {
					result.add(c);
					generateBoundary(c);
				}
			}
		}
		for (Method m : clazz.getDeclaredMethods()) {
			for (Class<?> c : m.getParameterTypes()) {
				if (!result.contains(c)) {
					result.add(c);
					generateBoundary(c);
				}
			}
			Class<?> c = m.getReturnType();
			if (!result.contains(c)) {
				result.add(c);
				generateBoundary(c);
			}
		}
	}
	
	public boolean contains(Class<?> clazz) {
		return classes.contains(clazz);
	}
	
	public Set<Class<?>> restrictTypeToBoundary(Class<?> clazz) {
		Set<Class<?>> result = new HashSet<Class<?>>();
		Set<Class<?>> toCheck = new HashSet<Class<?>>();
		toCheck.add(clazz);
		
		while (!toCheck.isEmpty()) {
			//get some element
			Class<?> c = toCheck.iterator().next();
			if (c.equals(java.lang.Object.class) || classes.contains(c)) {
				result.add(c);
				toCheck.remove(c);
			} else {
				toCheck.remove(c);
				Collections.addAll(toCheck, (Class<?>[])c.getInterfaces());
				toCheck.add(c.getSuperclass());
			}
		}
		return result;
	}

}
