package jbox.runtime;


import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import jbox.runtime.annotation.EmptySpec;
import jbox.runtime.spec.NoTransitionException;
import jbox.runtime.spec.SpecificationHandler;
import jbox.runtime.spec.SpecificationState;

import org.apache.log4j.Logger;

public class BoxI {
	private final Logger LOG = Logger.getLogger(BoxI.class);
	private static int currBoxNumbering = 0;
	
	private int boxId;
	private Class<?> interfaceClass;
	private Class<?> ownerClass;
	private int level;
	private final Boundary boundary;
	SpecificationHandler specHandler;
	SpecificationState specificationState;
	
	List<SpecificationHandler> specHandlers = new ArrayList<SpecificationHandler>();

	public BoxI(Class<?> interfaceClass, Class<?> handler, String spec, Class<?> ownerClass, BoxI parent) {
		super();
		boxId = currBoxNumbering++;
		if (parent == null) {
			this.level = 0;
		} else {
			this.level = parent.getLevel() + 1;
		}
		this.interfaceClass = interfaceClass;
		this.ownerClass = ownerClass;
		boundary = Boundary.getBoundary(interfaceClass);
		if (!handler.equals(EmptySpec.class)) {
			initSpecHandler(handler, spec);
		}
		LOG.info("New box " + toString() + " with interface " + interfaceClass.getName() + " created");
	}

	public String getName() {
		return "Box " + boxId;
	}

	@Override
	public String toString() {
		return "Box " + boxId;
	}

	public int getLevel() {
		return level;
	}

	public Class<?> getOwnerClass() {
		return ownerClass;
	}
	
	public Class<?> getInterfaceClass() {
		return interfaceClass;
	}
	
	private void initSpecHandler(Class<?> handler, String spec) {
		try {
			Method m = handler.getDeclaredMethod("create", BoxI.class, String.class);
			specHandler = (SpecificationHandler) m.invoke(null, this, spec);
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		specificationState = specHandler.initState();
	}

	public void transition(MsgSet messages) {
		LOG.info("Interbox message occured at box " + toString() + " with message set: ");
		LOG.info(messages);
		if (specHandler != null) {
			try {
				specificationState = specHandler.transition(messages, specificationState);
			} catch (NoTransitionException e) {
				LOG.fatal("Wrong transition according to spec");
				throw new JBoxException("Wrong transition according to spec: " + e.getMessage());
			}
		}
	}

	/*
	private void getSpec() {
		Class<BoxImplementation> boximplclazz;
		try {
			boximplclazz = (Class<BoxImplementation>) Class.forName("jbox.runtime.annotation.BoxImplementation", true, this.getClass().getClassLoader());
			Class.forName("jbox.runtime.annotation.BoxSpec", true, this.getClass().getClassLoader());
		} catch (ClassNotFoundException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
			System.exit(0);
			return;
		}

		System.out.println("here I was");
		try {
			Thread.sleep(200);
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		BoxImplementation bimpl = implClass.getAnnotation(BoxImplementation.class);

		Class<?> specClass = bimpl.spec();

		box = BoxSpecificationFactory.getBoxSpecification(specClass);

		automaton = new AutomatonInstance(box.getAutomaton());
	}



	public void setState(String state, String value) {
		//modify implState		
	}

	public Automaton getAutomaton() {
		return automaton.getAutomaton();
	}
	 */

	public boolean checkBoundary(Class<?> c) {
		//box.checkBoundary(c);
		if (level == 0) //globox
			return true;
		return boundary.contains(c);
	}

	public Set<Class<?>> restrictType(Class<?> type) {
		return boundary.restrictTypeToBoundary(type);
	}


}
