package jbox.runtime;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;
import java.util.WeakHashMap;

import jbox.agent.Agent;
import jbox.agent.IHandler;
import jbox.runtime.Msg.NoValidMessageException;
import jbox.runtime.annotation.EmptySpec;
import jbox.runtime.annotation.GloboxImpl;
import jbox.util.Pair;

import org.apache.log4j.Logger;

public class Handler implements IHandler {

	private final Logger LOG = Logger.getLogger(Handler.class);


	public final BoxI globox; //the global box

	public String nextObjectNewBoxIntfClass = null;
	public String nextObjectNewBoxHandler = null;
	public String nextObjectNewBoxSpec = null;

	public HashMap<Integer, BoxI> tempmap;
	public WeakHashMap<Object, BoxI> finalmap;

	public BoxI currentBox;

	public Stack<WeakReference<Object>> stack;

	public Handler() {
		super();
		LOG.debug("Handler initialized");
		globox = new BoxI(GloboxImpl.class, EmptySpec.class, "", GloboxImpl.class,  null);
		currentBox = globox;
		stack = new Stack<WeakReference<Object>>();
		tempmap = new HashMap<Integer, BoxI>();
		finalmap = new WeakHashMap<Object, BoxI>();
	}

	public BoxI get(Object key) {
		BoxI b = tempmap.get(new Integer(System.identityHashCode(key)));
		if (b==null) {
			b = finalmap.get(key);
		}
		return b;
	}

	private void putTemp(Object key, BoxI box) {
		LOG.debug("Putting to temp");
		tempmap.put(new Integer(System.identityHashCode(key)), box);
	}

	private void putFinal(Object key, BoxI box) {
		LOG.debug("Putting to final: " + key);
		tempmap.remove(new Integer(System.identityHashCode(key)));
		finalmap.put(key, box);
	}

	public void onNewObject(Object o) {
		BoxI registerBox = currentBox;
		if (nextObjectNewBoxIntfClass != null) {
			BoxI newbox = null;
			try {
				newbox = new BoxI(Class.forName(nextObjectNewBoxIntfClass), Class.forName(nextObjectNewBoxHandler), nextObjectNewBoxSpec, o.getClass(), currentBox);

				LOG.debug("creating new box " + newbox);

				//set parent
				putFinal(newbox, currentBox);

				registerBox = newbox;
				nextObjectNewBoxIntfClass = null;
			} catch (ClassNotFoundException e) {
				LOG.fatal("Box Interface class not found for: " + nextObjectNewBoxIntfClass);
				e.printStackTrace();
				return;
			}
		} 
		Class<?> dynamicType = o.getClass();
		LOG.debug("registering new obj "+ System.identityHashCode(o) + " of class " + dynamicType + " in box " + registerBox);

		if (!Agent.except.contains(dynamicType.getName())) {
			if (dynamicType.equals(java.lang.Object.class)) {
				putFinal(o, registerBox);
			} else {
				putTemp(o, registerBox);
			}
		}
	}

	public BoxI boxOf(Object o) {
		return get(o);
	}

	public boolean isIn(Object o, Object box) {
		if (o == null || box == null)
			return false;
		return get(o) == box;
	}

	public void onMethodEnter(Object newthis, Object[] args, String className, String methodName, String sig) {
		BoxI newBox = get(newthis);
		LOG.debug("onMethodEnter(instance)" + " " + className + "." + methodName + " in " + newBox);

		if (methodName.equals("<init>")) {
			try {
				if (methodName.equals("<init>")) {
					if (newthis.getClass().equals(Class.forName(className))) {

					} else {
						return;
					}
				}
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		if (newBox == null) {
			LOG.debug("object found which is not yet registered, assigning to " + currentBox + ": " + System.identityHashCode(newthis));
			newBox = globox;
			putFinal(newthis, newBox);
			//throw new RuntimeException("object found which is not yet registered");
		}
		if (newBox != currentBox) {
			BoxI oldBox = currentBox;
			
			//checkBoundary(newBox, sig);

			
			if (methodName.equals("<init>")) {
				List<BoxI> boxes = new ArrayList<BoxI>();
				int minlevel = transitionBoxes(oldBox, newBox, boxes);
				if (newBox.getLevel() != minlevel) {
					Set<Class<?>> singleSet = new HashSet<Class<?>>();
					singleSet.add(newthis.getClass());
					Set<Msg> messages = getMessagesAdaptedToBoundary(singleSet, methodName, sig, true, true);
					newBox.transition(new MsgSet(messages, null, args));
				}
			} else {
				//checkTypeAllowedInCurrentBox(newthis, oldBox, newBox, className, methodName, sig, true);
				List<Pair<BoxI,Boolean>> crossedBoxes = crossedBoxesInOut(oldBox, newBox);
				Set<Class<?>> currBoundary = new HashSet<Class<?>>();
				currBoundary.add(newthis.getClass());
				// box boundary map
				List<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>> list = new ArrayList<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>>();
				for (int i = 0; i < crossedBoxes.size(); i++) {
					Pair<BoxI,Boolean> pair = crossedBoxes.get(i);
					BoxI currBox = pair.getX();
					currBoundary = restrictTypeToBoundary(currBox, currBoundary);
					list.add(new Pair< Pair<BoxI,Boolean>, Set<Class<?>>>(pair, currBoundary));
				}
				
				for (int i = 0; i !=  list.size(); i = i + 1) {
					BoxI currBox = list.get(i).getX().getX();
					boolean b = list.get(i).getX().getY();
					Set<Msg> messages = getMessagesAdaptedToBoundary(currBoundary, methodName, sig, b, true);
					currBox.transition(new MsgSet(messages, null, args));
				}
			}
			
			LOG.info("box changed from " + currentBox + " to " + newBox);
			currentBox = newBox;
		}
		stack.push(new WeakReference<Object>(newthis));
	}

	private int transitionBoxes(BoxI start, BoxI end, List<BoxI> list) {
		List<BoxI> sl = new ArrayList<BoxI>();
		List<BoxI> el = new ArrayList<BoxI>();

		while (start != end) {
			if (start.getLevel() < end.getLevel()) {
				el.add(0, end);
				end = boxOf(end);
			} else
				if (start.getLevel() > end.getLevel()) {
					sl.add(start);
					start = boxOf(start);
				} else
					if (start.getLevel() == end.getLevel()) {
						sl.add(start);
						start = boxOf(start);
						el.add(0, end);
						end = boxOf(end);
					}
		}
		sl.add(start);
		sl.addAll(el);
		list.addAll(sl);
		return start.getLevel();
	}

	private List<BoxI> crossedBoxes(BoxI start, BoxI end) {
		List<BoxI> sl = new ArrayList<BoxI>();
		List<BoxI> el = new ArrayList<BoxI>();

		while (start != end) {
			if (start.getLevel() < end.getLevel()) {
				el.add(0, end);
				end = boxOf(end);
			} else
				if (start.getLevel() > end.getLevel()) {
					sl.add(start);
					start = boxOf(start);
				} else
					if (start.getLevel() == end.getLevel()) {
						sl.add(start);
						start = boxOf(start);
						el.add(0, end);
						end = boxOf(end);
					}
		}
		//sl.add(start);
		sl.addAll(el);
		return sl;
	}
	
	//the boolean stands for incoming
	private List<Pair<BoxI,Boolean>> crossedBoxesInOut(BoxI start, BoxI end) {
		List<Pair<BoxI,Boolean>> sl = new ArrayList<Pair<BoxI,Boolean>>();
		List<Pair<BoxI,Boolean>> el = new ArrayList<Pair<BoxI,Boolean>>();

		while (start != end) {
			if (start.getLevel() < end.getLevel()) {
				el.add(0, new Pair<BoxI,Boolean>(end, true));
				end = boxOf(end);
			} else
				if (start.getLevel() > end.getLevel()) {
					sl.add(new Pair<BoxI,Boolean>(start, false));
					start = boxOf(start);
				} else
					if (start.getLevel() == end.getLevel()) {
						sl.add(new Pair<BoxI,Boolean>(start, false));
						start = boxOf(start);
						el.add(0, new Pair<BoxI,Boolean>(end, true));
						end = boxOf(end);
					}
		}
		//sl.add(start);
		sl.addAll(el);
		return sl;
	}
	
	private Set<Class<?>> restrictTypeToBoundary(BoxI b, Set<Class<?>> clazzes) {
		Set<Class<?>> result = new HashSet<Class<?>>();
	    for (Class<?> c : clazzes) {
	    	result.addAll(b.restrictType(c));
	    }
		return result;
	}
	
	
	private Set<Class<?>> crossedRestrictions(List<BoxI> boxes, Class<?> clazz) {
		Set<Class<?>> result = new HashSet<Class<?>>();
		result.add(clazz);
		
		for (BoxI b : boxes) {
			result = restrictTypeToBoundary(b, result);
		}
		return result;
	}
	
	public void setState(String state, String value) {

	}

	public void onMethodExit(Object returnVal, Object oldthis, Object[] args, String className, String methodName, String sig) {
		LOG.debug(oldthis.getClass() + " " + className + "." + methodName);
		try {
			if (methodName.equals("<init>")) {
				//static type == dynamic type
				if (oldthis.getClass().equals(Class.forName(className))) {
					putFinal(oldthis, currentBox);
					if (oldthis.getClass().equals(java.lang.Object.class)) {
						putFinal(oldthis, currentBox);
						return;
					}

				} else {
					return;
				}

			}
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		LOG.debug("onMethodExit(instance)");
		stack.pop().get();
		BoxI newBox = null;
		BoxI oldBox = currentBox;
		if (stack.isEmpty()) {
			newBox = globox;
		} else {
			Object o = stack.pop().get();
			newBox = get(o);
			if (newBox == null) {
				LOG.fatal("new null box");
				throw new RuntimeException("new null box, should not occur");
			}
			stack.push(new WeakReference<Object>(o));
		}
		if (newBox != currentBox) {
			
			//checkBoundary(currentBox, sig);
			if (methodName.equals("<init>")) {
				List<BoxI> boxes = new ArrayList<BoxI>();
				int minlevel = transitionBoxes(newBox, oldBox, boxes);
				if (oldBox.getLevel() != minlevel) {
					Set<Class<?>> singleSet = new HashSet<Class<?>>();
					singleSet.add(oldthis.getClass());
					Set<Msg> messages = getMessagesAdaptedToBoundary(singleSet, methodName, sig, false, false);
					oldBox.transition(new MsgSet(messages, returnVal, args));
				}
			} else {
				//checkTypeAllowedInCurrentBox(oldthis, newBox, oldBox, className, methodName, sig, false);
				List<Pair<BoxI,Boolean>> crossedBoxes = crossedBoxesInOut(newBox, oldBox);
				Set<Class<?>> currBoundary = new HashSet<Class<?>>();
				currBoundary.add(oldthis.getClass());
				// box boundary map
				List<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>> list = new ArrayList<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>>();
				for (int i = 0; i < crossedBoxes.size(); i++) {
					Pair<BoxI,Boolean> pair = crossedBoxes.get(i);
					BoxI currBox = pair.getX();
					currBoundary = restrictTypeToBoundary(currBox, currBoundary);
					list.add(new Pair< Pair<BoxI,Boolean>, Set<Class<?>>>(pair, currBoundary));
				}
				
				for (int i = list.size()-1; i != -1; i = i - 1) {
					BoxI currBox = list.get(i).getX().getX();
					boolean b = list.get(i).getX().getY();
					Set<Msg> messages = getMessagesAdaptedToBoundary(currBoundary, methodName, sig, !b, false);
					currBox.transition(new MsgSet(messages, returnVal, args));
				}
			}
			LOG.info("box changed from " + currentBox + " to " + newBox);
			currentBox = newBox;
		}	
	}

	public void nextObjectNewBox(String intfClass, String specHandler, String spec) {		
		LOG.debug("nextObjectNewBox()");
		nextObjectNewBoxIntfClass = intfClass;
		nextObjectNewBoxHandler = specHandler;
		nextObjectNewBoxSpec = spec;
	}

	public void onFieldAccess(Object o) {
		if (o != null) {
			BoxI targetBox = get(o);
			if (targetBox == null) {
				return;
			}
			LOG.debug("Field of box " + targetBox + " in box " + currentBox + " accessed");

			if (targetBox != currentBox) {
				LOG.info("field access violated box conditions");
				throw new JBoxException("field access violated box conditions");
			}
		}
	}

	public void onMethodCall(Object o, String owner, String name, String desc) {
		//checkTypeAllowedInCurrentBox(o, owner, name, desc, true);
	}

	public void onMethodReturn(Object o, String owner, String name, String desc) {
		//checkTypeAllowedInCurrentBox(o, owner, name, desc, false);
	}


	private void checkTypeAllowedInCurrentBox(Object o, BoxI oldBox, BoxI newBox, String owner, String name, String desc, boolean isCall) {
		/*
		if (o == null) {
			LOG.fatal("About to call null object");
			return;
		}

		BoxI newBox = boxOf(o);

		if (newBox == null) {
			LOG.info("Call to object " + o + " of box " + newBox);
			return;
		}

		if (!currentBox.equals(newBox)) {
			//LOG.info("Transitioning from " + oldBox + " to " + newBox);
			
			List<Pair<BoxI,Boolean>> crossedBoxes = crossedBoxesInOut(oldBox, newBox);
			Set<Class<?>> currBoundary = new HashSet<Class<?>>();
			currBoundary.add(o.getClass());
			// box boundary map
			List<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>> list = new ArrayList<Pair< Pair<BoxI,Boolean>, Set<Class<?>>>>();
			for (int i = 0; i < crossedBoxes.size(); i++) {
				Pair<BoxI,Boolean> pair = crossedBoxes.get(i);
				BoxI currBox = pair.getX();
				currBoundary = restrictTypeToBoundary(currBox, currBoundary);
				list.add(new Pair< Pair<BoxI,Boolean>, Set<Class<?>>>(pair, currBoundary));
			}
			
			for (int i = isCall ? 0 : list.size()-1; i != (isCall ? list.size() : -1); i = i + (isCall ? 1 : -1)) {
				BoxI currBox = list.get(i).getX().getX();
				boolean b = list.get(i).getX().getY();
				boolean direction = isCall ? b : !b;
				Set<Msg> messages = getMessagesAdaptedToBoundary(currBoundary, name, desc, direction, isCall);
				currBox.transition(messages);
			}
			
		}*/
	}
	
	private Set<Msg> getMessagesAdaptedToBoundary(Set<Class<?>> boundary, String methodName, String sig, boolean incoming, boolean call) {
		Set<Msg> messages = new HashSet<Msg>();
		for (Class<?> c : boundary) {
			try {
				Msg m = new Msg(c, methodName, sig, incoming, call);
				messages.add(m);
			} catch (NoValidMessageException e) {}
		}
		if (messages.isEmpty()) {
			//throw exception here
		}
		return messages;
	}

	public void onCast(Object o, String type) {
		try {
			Class<?> dType = o.getClass();
			Class<?> clazz = Class.forName(type);
			BoxI destBox = get(o);
			Set<Class<?>> clazzes = crossedRestrictions(crossedBoxes(currentBox, destBox), dType);
			//find one which is a supertype of clazz
			for (Class<?> c : clazzes) {
				if (c.isAssignableFrom(clazz)) {
					//check ok
					return;
				}
			}
			throw new JBoxException("Cast to class " + type + " failed because of box signature");
		} catch (ClassNotFoundException e) {
			LOG.fatal("Class not found which should be there: " + type);
			//e.printStackTrace();
		}
	}

}

