package jbox.runtime;

import java.lang.reflect.Member;

import org.objectweb.asm.Type;

public class Msg {
	
	private Member member; //if it is a constructor we only need to know the parameter types
	private boolean incoming;
	private boolean call;
	
	public Msg(Class<?> c, String methodName, String sig, boolean incoming, boolean call)
	  throws NoValidMessageException {
		Type[] paramTypes = Type.getArgumentTypes(sig);
		Class<?>[] parameterTypes = new Class[paramTypes.length];
		for (int i = 0; i < paramTypes.length; i++) {
			try {
				parameterTypes[i] = Class.forName(paramTypes[i].getClassName());
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
		try {
			if (methodName.equals("<init>")) {
				member = c.getConstructor(parameterTypes);
			} else {
				
				member = c.getDeclaredMethod(methodName, parameterTypes);
			}
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			throw new NoValidMessageException();
		}

		this.incoming = incoming;
		this.call = call;
	}
	
	
	
	public static class NoValidMessageException extends Exception {
		
	}



	public Member getMember() {
		return member;
	}



	public boolean isIncoming() {
		return incoming;
	}



	public boolean isCall() {
		return call;
	}
	
	@Override
	public String toString() {
		String arrow;
		if (incoming)
			arrow = "->";
		else
			arrow = "<-";
		String callSign;
		if (call)
			callSign = "c";
		else
			callSign = "r";
		return callSign + arrow + " " + member;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Msg) {
			Msg msg = (Msg)o;
			return msg.member.equals(member) && msg.incoming == incoming && msg.call == call;
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return member.hashCode() + (incoming ? 0 : 1) + (call? 0 : 2);
	}
}