package jcs.checker;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import jbox.runtime.Msg;
import jcs.ast.gen.Call;
import jcs.ast.gen.CallNd;
import jcs.ast.gen.MethodDeclaration;
import jcs.ast.gen.MethodDeclarationNd;
import jcs.ast.gen.Parameter;
import jcs.types.ConcreteType;
import jcs.types.Environment;
import jcs.types.IntersectionType;
import jcs.types.Type;

//Types should be in box signature
public class AbstractMsg {
	//int label;
	List<Type> calleeType = new ArrayList<Type>();
	String methodName;
	List<Type> parameterTypes = new ArrayList<Type>();
	
	boolean isCall;
	boolean isIncoming;
	//some more here
	
	
	public AbstractMsg(Environment env, CallNd call, boolean isIncoming) {
		Call c = call.term();
		this.calleeType.add(env.getType(c.type()));
		
		this.methodName = c.methodName().child0();
		
		for (Parameter p : c.params()) {
			this.parameterTypes.add(env.getType(p.type()));
		}
		
		this.isCall = !isIncoming;
		this.isIncoming = isIncoming;
	}
	
	public AbstractMsg(Environment env, MethodDeclarationNd method, boolean isIncoming) {
		MethodDeclaration m = method.term();
		for (jcs.ast.gen.Type t : method.parent().parent().term().types()) {
			this.calleeType.add(env.getType(t));
		}
		
		this.methodName = m.id().child0();
		
		for (Parameter p : m.params()) {
			this.parameterTypes.add(env.getType(p.type()));
		}
		
		this.isCall = isIncoming;
		this.isIncoming = isIncoming;
	}
	
	public boolean matches(Set<Msg> messages) {
		//TODO: look out for constructors
		if (messages.size() == 1) {
			return matches(messages.iterator().next());
		}
		
	    for (Msg m : messages) {
	    	if (!matches(m)) {
	    		return false;
	    	}
	    }
		return true;
	}
	
	
	
	private boolean matches(Msg m) {
		//System.out.println("Trying to match " + m);
		//System.out.println("against " + this);
		if (m.isCall() != isCall || m.isIncoming() != isIncoming) {
			return false;
		}
		if (!m.getMember().getName().equals(methodName)) {
			return false;
		}
		boolean foundCallee = false;
		for (Type t : calleeType) {
			if (t.matches(m.getMember().getDeclaringClass())) {
				foundCallee = true;
				break;
			}
		}
		if (!foundCallee) {
			return false;
		}
		
		return true;
	}

	protected boolean matchesType(Class<?> cType, Type aType) {
		if (aType instanceof IntersectionType) {
			
		}
		if (aType instanceof ConcreteType) {
			
		}
		
		return false;
	}

	public boolean isIncoming() {
		return isIncoming;
	}
	
	public boolean isCall() {
		return isCall;
	}
	
	@Override
	public String toString() {
		String arrow;
		if (isIncoming)
			arrow = "->";
		else
			arrow = "<-";
		String callSign;
		if (isCall)
			callSign = "c";
		else
			callSign = "r";
		return callSign + arrow + " " + methodName + "()";
	}
}
