package jcs.checker;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;
import java.util.Map;

import org.sat4j.minisat.SolverFactory;

import jbox.runtime.Msg;
import jbox.runtime.spec.NoTransitionException;

public class GPDA {
	Set<Integer> labels;
	
	boolean DEBUG = false;
	
	//basically the next function
	private HashMap<Integer, Set<Destination>> next = new HashMap<Integer, Set<Destination>>();
	
	
	//for pretty printing
	private HashMap<Integer, Guard> guardsMap = new HashMap<Integer, Guard>(); 
	
	public void addTransitions(Integer from, Set<Destination> destination) {
		next.put(from, destination);
		for (Destination d : destination) {
			if (!d.msg.isIncoming()) {
				if (!guardsMap.containsKey(d.label)) {
					guardsMap.put(d.label, d.getGuard());
				}
			}
		}
		
		if (DEBUG) {
			System.out.println("Adding transitions from " + from);
			for (Destination d : destination) {
				System.out.println("to label " + d.getLabel());
				System.out.println("guard " + d.getGuard().problem);
			}
		}
	}
	
	
	public UnionState transition(Set<Msg> messages, UnionState currentState) throws NoTransitionException {
		//TODO: Constructors:
		if (messages.size() == 1 && messages.iterator().next().getMember() instanceof Constructor<?>) {
			return currentState;
		}
		UnionState result = new UnionState();
		for (State state : currentState.states) {
			//first filter next possible nodes
			Set<Destination> succLabels = next.get(state.label);
			for (Destination dest : succLabels) {
				//then filter message matches and check labels / stack
				if (dest.msg.matches(messages) && isStackValid(dest.getMessage(), dest.label, state)) {
					//System.out.println("come here2");
					//now evaluate conditions
					if (checkConditions(state.guard, dest.guard)) {
						//System.out.println("come here3");
						//construct result
						State match = new State();
						match.label = dest.label; //from dest.msg
						if (!dest.msg.isIncoming()) {
							match.guard = dest.guard;
						} else {
							match.guard = new Guard();
						}
						match.stack = new Stack<Integer>(); //
						match.stack.addAll(state.stack);
						//now push or pop
						modifyStack(dest.getMessage(), dest.label, match.stack);
						result.states.add(match);
					}				
				}
			}
			
			
		}
		//at least one must transition
		if (result.states.isEmpty()) {
			throw new NoTransitionException();
		}
		return result;
	}
	
	private void modifyStack(AbstractMsg m, int label, Stack<Integer> stack) {
		if (m.isIncoming() && !m.isCall() || !m.isIncoming() && !m.isCall()) {
			stack.pop();
		} else {
			stack.push(label);
		}
	}


	private boolean checkConditions(Guard g1, Guard g2) {
		Guard g = g1.and(g2);
		if (g.problem == null) {
			return true;
		}
		try {
			List<?> solutions = g.problem.findModel(SolverFactory.newMiniSAT());
			return solutions.size() != 0;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}


	private boolean isStackValid(AbstractMsg m, int label, State state) {
		if (!m.isCall()) {
			//System.out.println("Was " + state.stack.peek() + " should be " + label);
			return label / 2 == state.stack.peek() / 2;
		}
		return true;
	}
	
	
	public String toDot() {
		StringBuilder b = new StringBuilder();
		b.append("digraph G ");
		b.append("{\n");
		b.append("nodesep=1.5;\n");
		for (Map.Entry<Integer, Set<Destination>> e : next.entrySet()) {
			for (Destination d : e.getValue()) {
				/*if (guardsMap.containsKey(e.getKey())) {
					b.append(e.getKey() + " " + guardsMap.get(e.getKey()));
				} else {
					b.append(e.getKey());
				}*/
				b.append(nodeName(e.getKey()));
				b.append(" -> ");
				b.append(nodeName(d.label));
				b.append(" [ ");
				b.append("label = \"");
				b.append(d.getMessage().methodName);
				b.append("," + (d.getMessage().isCall() ? "+" : "-") + "L" + (d.label / 2));
				b.append(",\\n" + d.guard);
				b.append("\"");
				b.append(" ,color = \"");
				b.append(d.getMessage().isIncoming() ? "red" : "blue");
				b.append("\"");
				b.append(" ,fontsize = \"12\"");
				b.append(" ,style = \"");
				b.append(d.getMessage().isCall() ? "bold" : "filled");
				b.append("\"");
				b.append(" ]");
				b.append("\n");
			}
		}
		b.append("}");
		return b.toString();
		
	}
	
	private String nodeName(Integer id) {
		if (guardsMap.containsKey(id)) {
			return "\"" + "N" + id + " (" + guardsMap.get(id) + ")\"";
		} else {
			return "N"+id.toString();
		}
	}
	

}
