package jcs.checker;


import jcs.ast.gen.AndExpression;
import jcs.ast.gen.EqExpression;
import jcs.ast.gen.Expression;
import jcs.ast.gen.Identifier;
import jcs.ast.gen.Literal;
import jcs.ast.gen.NotExpression;
import jcs.ast.gen.OrExpression;
import jcs.types.Environment;
import jcs.types.StateType;

import org.sat4j.core.VecInt;
import org.sat4j.minisat.SolverFactory;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.TimeoutException;

import positronic.satisfiability.elements.BitFixer;
import positronic.satisfiability.elements.BooleanVariable;
import positronic.satisfiability.elements.ClauseDenier;
import positronic.satisfiability.elements.Conjunction;
import positronic.satisfiability.elements.Disjunction;
import positronic.satisfiability.elements.IBooleanVariable;
import positronic.satisfiability.elements.IProblem;
import positronic.satisfiability.elements.ProblemDenier;

import static org.sat4j.core.LiteralsUtils.*;

/**
 * We have to transform formula into CNF before we can use it as input for the SAT checker
 * This is done using the positronic library
 * @author welsch
 *
 */
public class StateTransformer {
	
	private Environment env;
	
	
	
	
	
	
	
	public StateTransformer(Environment env) {
		super();
		this.env = env;
	}


	public static void main(String[] args) throws Exception {
		ISolver solver = SolverFactory.instance().lightSolver();
		solver.setTimeout(3600); //1 hour
		solver.newVar(2);
		/*
		solver.addClause(new VecInt(new int[] {1,2}));
		solver.addClause(new VecInt(new int[] {-1,-2}));
		solver.addClause(new VecInt(new int[] {1,-2}));
		solver.addClause(new VecInt(new int[] {-1,2}));
		*/
		solver.addClause(new VecInt(new int[] {-1}));
		solver.addClause(new VecInt(new int[] {-1}));
		
		/*int[] model = solver.findModel();
		
		
		for (int i = 0; i < model.length; i++) {
			System.out.println(var(model[i]));
		}
		*/
		
		
		System.out.println(solver.isSatisfiable());
		
		System.out.println(posLit(2) + " " + (2 << 1));
		System.out.println(negLit(2) + " " + neg(posLit(2)));
		
		System.out.println(var(4));
		System.out.println(var(5));
		
		
		IBooleanVariable x = BooleanVariable.getBooleanVariable("x");

	    IBooleanVariable y = BooleanVariable.getBooleanVariable("y");

	    IBooleanVariable z = BooleanVariable.getBooleanVariable("z");

	    IProblem bfx1=new BitFixer(x,true);

	    IProblem bfy1=new BitFixer(x,false);
	    
	    IProblem bfx2=new BitFixer(y,true);

	    IProblem bfy2=new BitFixer(y,false);

	    
	    IProblem p1 = new ProblemDenier(new Conjunction(bfx1,bfy1));
	    //p1 = new Conjunction(p1, new Conjunction(bfx1,bfy1));
	    
	    //catch org.sat4j.specs.ContradictionException (while constructing)
	    
	    //ProblemDenier
	    
	    System.out.println(p1);
	    
	    java.util.List v1=p1.findModel(SolverFactory.newMiniSAT());

	    
	    System.out.println(v1);



	}
	
	
	public Guard toGuard(Expression e) {
		Guard result = new Guard();
		
		if (e instanceof OrExpression) {
			
		}
		if (e instanceof AndExpression) {
			
		}
		if (e instanceof EqExpression) {
			EqExpression eq = e.<EqExpression>cast();
			if (eq.e1() instanceof Identifier
					&& eq.e2() instanceof Identifier) {
				Identifier i1 = eq.e1().<Identifier>cast();
				Identifier i2 = eq.e2().<Identifier>cast();
				StateType stype = env.getStateType(i1.child0());
				if (stype.hasValue(i2.child0())) {
					result.problem = buildProblem(stype, i2.child0());
					return result;
				}
				
			}
		}
		if (e instanceof NotExpression) {
			
		}
		if (e instanceof Literal) { //e.g. true false 1.0 etc.
			
		}
		if (e instanceof Identifier) {
			
		}
		
		throw new IllegalArgumentException("Not implemented yet");
	}
	
	private IProblem buildProblem(StateType stype, String value) {
		try {
			IProblem[] l = new IProblem[stype.getValues().size()];
			int i = 0;
			for (String v : stype.getValues()) {
				//should check if value is not in other stype, then make prefix
				//IBooleanVariable x = BooleanVariable.getBooleanVariable(stype.getId() + "$" + v);
				IBooleanVariable x = BooleanVariable.getBooleanVariable(v);
				boolean flag = v.equals(value);
				IProblem p = new BitFixer(x, flag);
				l[i] = p;
				i++;
			}
			
			return new Conjunction(l);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		
		return null;
	}
	

	
	

}
