package xtc.lang.jeannie;

import java.io.IOException;

import xtc.lang.jeannie.Debugger.DebugerControlStatus;
import xtc.lang.jeannie.DebuggerAstAnalyzer.Language;
import xtc.lang.jeannie.JeanniePrinter.TransitionKind;
import xtc.tree.GNode;
import xtc.tree.Visitor;

public final class DebuggerInterpreter extends Visitor {
  private final Debugger _dbg;
  
  DebuggerInterpreter(final Debugger dbg) {
    _dbg = dbg;
  }
  
  /** Visit an AdditiveExpression = Expression ("+" / "-") Expression. */
  public final void visitAdditiveExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit an AddressExpression = Expression. */
  public final void visitAddressExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit Arguments = Expression*. */
  public final void visitArguments(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit an AssignmentExpression = Expression ("=" / "+=" / "-=" / "*=" / "/=" / "%=" / "&lt;&lt;=" / "&gt;&gt;=" / "&amp;=" / "^=" / "|=") Expression. */
  public final void visitAssignmentExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a BreakClassCommand = QualifiedIdentifier LineNumber. */
  public final void visitBreakClassCommand(final GNode n) {    
    String className = n.getNode(0).getString(0);
    int lineNumber = Integer.parseInt( n.getString(1));
    _dbg.debuggerCommand.executeStopAt(className, lineNumber);
  }

  /** Visit a BreakFileCommand = FileName LineNumber. */
  public final void visitBreakFileCommand(final GNode n) {
    String fileName = n.getString(0);
    int lineNumber = Integer.parseInt(n.getString(1));
    _dbg.debuggerCommand.executeBreak(fileName, lineNumber);
  }
  
  /** Visit a C2jCommand = (no children). */
  public final void visitC2jCommand(final GNode n) {
    _dbg.n2j();
  }

  /** Visit a CallExpression = [Expression] null MethodName Arguments. */
  public final void visitCallExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a CInJavaExpression = C.UnaryExpression. */
  public final void visitCInJavaExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a ContinueCommand = (no children). */
  public final void visitContinueCommand(final GNode n) {
    _dbg.cont();
  }

  /** Visit a DeleteCommand = IntegerLiteral. */
  public final void visitDeleteCommand(final GNode n) {
    int id = Integer.parseInt(n.getNode(0).getString(0));
    _dbg.debuggerCommand.executeDelete(id);
  }

  /** Visit a DirectComponentSelection = Expression String. */
  public final void visitDirectComponentSelection(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a DownCommand = IntegerLiteral. */
  public final void visitDownCommand(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a EqualityExpression = Expression ("==" / "!=") Expression. */
  public final String visitEqualityExpression(final GNode n) {
    assert false : "not yet implemented";
    //maybe, use meta-variables or some other temporary storage of pointers in Java
    final DebuggerAstAnalyzer.Language language = DebuggerAstAnalyzer.getNodeLanguage(n);
    final String left = (String) dispatch(n.getNode(0));
    final String operator = n.getString(1);
    final String right = (String) dispatch(n.getNode(2));
    if (language == DebuggerAstAnalyzer.Language.C) {
      //TD gdb evaluate expression
    } else {
      //TD jdb evaluate expression
    }
    return null;
  }
  


  /** Visit a Expression = Expression ("=" / "+=" / "-=" / "*=" / "/=" / "&amp;=" / "|=" / "^=" / "%=" / "&lt;&lt;=" / "&gt;&gt;=" / "&gt;&gt;&gt;=") Expression. */
  public final void visitExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit ExpressionList = Expression+. */
  public final void visitExpressionList(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a FunctionCall = Expression [ExpressionList]. */
  public final void visitFunctionCall(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a GdbCommand = RestOfLine. */
  public final void visitGdbCommand(final GNode n) {
    String cmd = n.getString(0);
    _dbg.debuggerCommand.executeGDBCommand(cmd);
  }

  /** Visit a HelpCommand = (no children). */
  public final void visitHelpCommand(final GNode n) {
    _dbg.printCommandHelp();
  }

  /** Visit an IndirectionExpression = Expression. */
  public final void visitIndirectionExpression(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a InfoBreakCommand = (no children). */
  public final void visitInfoBreakCommand(final GNode n) {
    _dbg.debuggerCommand.executeInfoBreak();
  }

  /** Visit a InfoWatchCommand = (no children). */
  public final void visitInfoWatchCommand(final GNode n) {
    _dbg.debuggerCommand.executeInfoWatch();
  }
  
  /** Visit an InitJCommand = (no children). */
  public final void visitInitJCommand(final GNode n) {
    _dbg.preparingDebuggerSwitching();
  }
  
  /** Visit an IntegerConstant = String. */
  public final void visitIntegerConstant(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit an IntegerLiteral = String. */
  public final void visitIntegerLiteral(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a J2cCommand = (no children). */
  public final void visitJ2cCommand(final GNode n) {
    _dbg.j2n();
  }

  /** Visit a JavaInCExpression = Java.UnaryExpression. */
  public final void visitJavaInCExpression(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a JdbCommand = RestOfLine. */
  public final void visitJdbCommand(final GNode n) {
    String cmd = n.getString(0);
    _dbg.debuggerCommand.executeJDBCommand(cmd);
  }
  
  /** Visit a JRetCommand = (no children). */
  public final void visitJRetCommand(final GNode n) {
    _dbg.jret();
  }

  /** Visit a ListCommand = (no children). */
  public final void visitListCommand(final GNode n) {
    _dbg.debuggerCommand.executeList();
  }
  
  /** Visit a LocalsCommand = (no children). */
  public final void visitLocalsCommand(final GNode n) {
    _dbg.debuggerCommand.executeLocals();
  }
  
  /** Visit a MetaVariable = Identifier. */
  public final void visitMetaVariable(final GNode n) {
    assert false : "not yet implemented";
  }

  /** Visit a NextCommand = (no children). */
  public final void visitNextCommand(final GNode n) {
    _dbg.debuggerCommand.executeNext();
  }

  /** Visit a NextCommand = (no children). */
  public final void visitRunCommand(final GNode n) {
    _dbg.run();
  }

  /** Visit an StringLiteral = (no children). */
  public final void visitNullLiteral(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a PrimaryIdentifier = String. */
  public void visitPrimaryIdentifier(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a PrintCExpressionCommand = C.Expression. */
  public final void visitPrintCExpressionCommand(final GNode n) {
    _dbg.debuggerCommand.executePrint(n);
  }

  /** Visit a PrintJavaExpressionCommand = Java.Expression. */
  public final void visitPrintJavaExpressionCommand(final GNode n) {
    _dbg.debuggerCommand.executePrint(n);
  }

  /** Visit a QualifiedIdentifier = Identifier+. */
  public final String visitQualifiedIdentifier(final GNode n) {
    return Utilities.qualifiedIdentifierToString(n);
  }

  /** Visit a SelectionExpression = Expression Identifier. */
  public final void visitSelectionExpression(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a StatCommand = (no children). */
  public final void visitStatCommand(final GNode n) {
    _dbg.printStatus();
  }

  /** Visit a StepCommand = (no children). */
  public final void visitStepCommand(final GNode n) {
    _dbg.debuggerCommand.executeStep();
  }
  
  /** Visit an StringConstant = String. */
  public final void visitStringConstant(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit an StringLiteral = String. */
  public final void visitStringLiteral(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a SubscriptExpression = Expression Expression. */
  public final void visitSubscriptExpression(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a ThisExpression = [Expression]. */
  public final void visitThisExpression(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit an UpCommand = IntegerLiteral. */
  public final void visitUpCommand(final GNode n) {
    int steps = Integer.parseInt( n.getNode(0).getString(0));
    _dbg.debuggerCommand.executeUp(steps);
  }

  /** Visit a WatchCExpressionCommand = C.Expression. */
  public final void visitWatchCExpressionCommand(final GNode n) {
    assert false : "not yet implemented";
  }
  
  /** Visit a WatchJavaFieldCommand = [WatchKind] QualifiedIdentifier. */
  public final void visitWatchJavaFieldCommand(final GNode n) {
    String accessType = n.getString(0);
    String className = n.getNode(1).getString(0);
    String fieldName = n.getNode(1).getString(1);
    _dbg.debuggerCommand.executeWatch(accessType, className, fieldName);
  }
  
  /** Visit a WhereCommand = (no children). */
  public final void visitWhereCommand(final GNode n) {
    _dbg.debuggerCommand.executeWhere();
  }
}
