package xtc.lang.jeannie;

import xtc.lang.jeannie.DebuggerExpression.TargetSourceLanguage;

public interface DebuggerSymbolMapper {
  SourceFileAndLine getCurrentSourceFileAndLine();
  
  VariableRemapEntry lookUpVariableRemap(String variable, String sourceFile, int sourceLineNumber);

  /**
   * A source-level view of the current position. This is a
   * <source file, source line> pair.
   */
  static class SourceFileAndLine {

    /**
     * The source File.
     */
    private final String sourceFile;

    /**
     * The source line.
     */
    private final int sourceLine;

    /**
     * @param sourceFile The source file.
     * @param sourceLine The source line.
     */
    SourceFileAndLine(String sourceFile, int sourceLine) {
      this.sourceFile = sourceFile;
      this.sourceLine = sourceLine;
    }

    /**
     * @return The source file.
     */
    public String getSourceFile() {
      return sourceFile;
    }

    /**
     * @return The source line.
     */
    public int getSourceLine() {
      return sourceLine;
    }
  }

  /**
   * A source level variable entry.
   */
  static class VariableRemapEntry {

    /**
     * Variable Remap for the Jeannie source-to-source transformation.
     */
    final int startLine;
    final int startColumn;
    final int endLine;
    final int endColumn;
    final TargetSourceLanguage targetLanguage;
    final String sourceVariableName;
    final String targetVariableName;

    /**
     * @param startLine The start line in the target source file.
     * @param startColumn The start column in the target source file.
     * @param endLine The end line in the target source file.
     * @param endColumn The end column in the target source file.
     * @param targetLanguage The target language (C or Java).
     * @param sourceVariableName The variable name in the source language.
     * @param targetVariableName The variable name in the target language.
     */
    VariableRemapEntry(int startLine, int startColumn, int endLine,
        int endColumn, TargetSourceLanguage targetLanguage,
        String sourceVariableName, String targetVariableName) {
      this.startLine = startLine;
      this.startColumn = startColumn;
      this.endLine = endLine;
      this.endColumn = endColumn;
      this.targetLanguage = targetLanguage;
      this.sourceVariableName = sourceVariableName;
      this.targetVariableName = targetVariableName;    
    }

    /**
     * Generate the variable expression in the target language environment such
     * that jdb or gdb can recognize the expression.
     * 
     * @return jdb or gdb expression for the variable.
     */
    String targetLanguageExpression() {
      StringBuffer sb = new StringBuffer();
      switch(targetLanguage) {
      case JAVA:
        sb.append("jEnv.").append(targetVariableName);
        break;
      case C:
        sb.append("cEnv.").append(targetVariableName);
        break;
      default:
        assert false;
        break;
      }
      return sb.toString();
    }
  }
}
