package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface Assert extends jcs.ast.gen.Statement, KatjaTuple {

    //----- methods of Assert -----

    public jcs.ast.gen.Expression expr();
    public jcs.ast.gen.Assert replaceExpr(jcs.ast.gen.Expression expr);
    public jcs.ast.gen.Expression get(int i);
    public int size();
    public jcs.ast.gen.Assert replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E;

    //----- nested classes of Assert -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitExpression(jcs.ast.gen.Expression term) throws E;
        public void visitOrExpression(jcs.ast.gen.OrExpression term) throws E;
        public void visitAndExpression(jcs.ast.gen.AndExpression term) throws E;
        public void visitEqExpression(jcs.ast.gen.EqExpression term) throws E;
        public void visitNotExpression(jcs.ast.gen.NotExpression term) throws E;
        public void visitLiteral(jcs.ast.gen.Literal term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitBoolValue(jcs.ast.gen.BoolValue term) throws E;
        public void visitIntValue(jcs.ast.gen.IntValue term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitAssert(jcs.ast.gen.Assert term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.Assert.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.Expression.Switch<Object, E> variantVisit$Expression = new jcs.ast.gen.Expression.Switch<Object, E>() { public final Object CaseOrExpression(jcs.ast.gen.OrExpression term) throws E { visitOrExpression(term); return null; } public final Object CaseAndExpression(jcs.ast.gen.AndExpression term) throws E { visitAndExpression(term); return null; } public final Object CaseEqExpression(jcs.ast.gen.EqExpression term) throws E { visitEqExpression(term); return null; } public final Object CaseNotExpression(jcs.ast.gen.NotExpression term) throws E { visitNotExpression(term); return null; } public final Object CaseLiteral(jcs.ast.gen.Literal term) throws E { visitLiteral(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseBoolValue(jcs.ast.gen.BoolValue term) throws E { visitBoolValue(term); return null; } public final Object CaseIntValue(jcs.ast.gen.IntValue term) throws E { visitIntValue(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitExpression(jcs.ast.gen.Expression term) throws E {
            term.Switch(variantVisit$Expression);
        }
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.Assert {

        //----- attributes of Impl -----

        private jcs.ast.gen.Expression _expr = null;

        //----- methods of Impl -----

        public jcs.ast.gen.Expression expr() {
            return _expr;
        }

        public jcs.ast.gen.Assert replaceExpr(jcs.ast.gen.Expression expr) {
            return replace(0, expr);
        }

        Impl(jcs.ast.gen.Expression expr) {
            if(expr == null)
                throw new IllegalArgumentException("constructor of sort Assert invoked with null parameter expr");

            this._expr = expr;
        }

        public jcs.ast.gen.Expression get(int i) {
            int ith = i;

            if(ith < 0) ith += 1;

            switch(ith) {
                case 0: return _expr;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort Assert invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort Assert invoked with too large parameter "+i+", sort has only 1 components");
                    }
            }
        }

        public int size() {
            return 1;
        }

        public jcs.ast.gen.Assert replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort Assert invoked with negative parameter "+pos);
            if(pos >= 1)
                throw new IllegalArgumentException("replace on sort Assert invoked with too large parameter "+pos+", sort has only 1 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.Expression))
                throw new IllegalArgumentException("replace on sort Assert invoked with term of incorrect sort, Expression expected");

            return (jcs.ast.gen.Assert) JCS.unique(new jcs.ast.gen.Assert.Impl(
                pos == 0 ? (jcs.ast.gen.Expression) term : _expr
            ));
        }

        public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseAssert(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.Assert");
            builder.append("( ");
            _expr.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("Assert");
            builder.append("( ");
            _expr.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "Assert";
        }
    }
}

