package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface Block extends jcs.ast.gen.Statement, KatjaList<jcs.ast.gen.Statement> {

    //----- methods of Block -----

    public jcs.ast.gen.Block add(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block addAll(KatjaList<? extends jcs.ast.gen.Statement> list);
    public jcs.ast.gen.Block remove(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block removeAll(KatjaList<? extends jcs.ast.gen.Statement> list);
    public jcs.ast.gen.Block appBack(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block appFront(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block conc(KatjaList<? extends jcs.ast.gen.Statement> list);
    public jcs.ast.gen.Block front();
    public jcs.ast.gen.Block back();
    public jcs.ast.gen.Block reverse();
    public jcs.ast.gen.Block sublist(int from, int to);
    public jcs.ast.gen.Block replace(int ith, Object element);
    public jcs.ast.gen.Block toSet();
    public jcs.ast.gen.Block setAdd(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block setRemove(jcs.ast.gen.Statement element);
    public jcs.ast.gen.Block setUnion(KatjaList<? extends jcs.ast.gen.Statement> list);
    public jcs.ast.gen.Block setIntersection(KatjaList<? extends jcs.ast.gen.Statement> list);
    public jcs.ast.gen.Block setWithout(KatjaList<? extends jcs.ast.gen.Statement> list);
    public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E;

    //----- nested classes of Block -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitStatement(jcs.ast.gen.Statement term) throws E;
        public void visitOr(jcs.ast.gen.Or term) throws E;
        public void visitLoop(jcs.ast.gen.Loop term) throws E;
        public void visitBlock(jcs.ast.gen.Block term) throws E;
        public void visitAssume(jcs.ast.gen.Assume term) throws E;
        public void visitAssert(jcs.ast.gen.Assert term) throws E;
        public void visitCall(jcs.ast.gen.Call term) throws E;
        public void visitEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E;
        public void visitBreak(jcs.ast.gen.Break term) throws E;
        public void visitExpression(jcs.ast.gen.Expression term) throws E;
        public void visitType(jcs.ast.gen.Type term) throws E;
        public void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitParameterList(jcs.ast.gen.ParameterList term) throws E;
        public void visitOrExpression(jcs.ast.gen.OrExpression term) throws E;
        public void visitAndExpression(jcs.ast.gen.AndExpression term) throws E;
        public void visitEqExpression(jcs.ast.gen.EqExpression term) throws E;
        public void visitNotExpression(jcs.ast.gen.NotExpression term) throws E;
        public void visitLiteral(jcs.ast.gen.Literal term) throws E;
        public void visitBoolValue(jcs.ast.gen.BoolValue term) throws E;
        public void visitIntValue(jcs.ast.gen.IntValue term) throws E;
        public void visitWildcardOne(jcs.ast.gen.WildcardOne term) throws E;
        public void visitWildcardAny(jcs.ast.gen.WildcardAny term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitIntersectionType(jcs.ast.gen.IntersectionType term) throws E;
        public void visitBasicType(jcs.ast.gen.BasicType term) throws E;
        public void visitIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitParameter(jcs.ast.gen.Parameter term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.Block.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.Expression.Switch<Object, E> variantVisit$Expression = new jcs.ast.gen.Expression.Switch<Object, E>() { public final Object CaseOrExpression(jcs.ast.gen.OrExpression term) throws E { visitOrExpression(term); return null; } public final Object CaseAndExpression(jcs.ast.gen.AndExpression term) throws E { visitAndExpression(term); return null; } public final Object CaseEqExpression(jcs.ast.gen.EqExpression term) throws E { visitEqExpression(term); return null; } public final Object CaseNotExpression(jcs.ast.gen.NotExpression term) throws E { visitNotExpression(term); return null; } public final Object CaseLiteral(jcs.ast.gen.Literal term) throws E { visitLiteral(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseBoolValue(jcs.ast.gen.BoolValue term) throws E { visitBoolValue(term); return null; } public final Object CaseIntValue(jcs.ast.gen.IntValue term) throws E { visitIntValue(term); return null; } };
        private final jcs.ast.gen.IdentifierOpt.Switch<Object, E> variantVisit$IdentifierOpt = new jcs.ast.gen.IdentifierOpt.Switch<Object, E>() { public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E { visitIdentifierNone(term); return null; } };
        private final jcs.ast.gen.Statement.Switch<Object, E> variantVisit$Statement = new jcs.ast.gen.Statement.Switch<Object, E>() { public final Object CaseOr(jcs.ast.gen.Or term) throws E { visitOr(term); return null; } public final Object CaseLoop(jcs.ast.gen.Loop term) throws E { visitLoop(term); return null; } public final Object CaseBlock(jcs.ast.gen.Block term) throws E { visitBlock(term); return null; } public final Object CaseAssume(jcs.ast.gen.Assume term) throws E { visitAssume(term); return null; } public final Object CaseAssert(jcs.ast.gen.Assert term) throws E { visitAssert(term); return null; } public final Object CaseCall(jcs.ast.gen.Call term) throws E { visitCall(term); return null; } public final Object CaseEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E { visitEmptyStatement(term); return null; } public final Object CaseBreak(jcs.ast.gen.Break term) throws E { visitBreak(term); return null; } };
        private final jcs.ast.gen.Type.Switch<Object, E> variantVisit$Type = new jcs.ast.gen.Type.Switch<Object, E>() { public final Object CaseWildcardOne(jcs.ast.gen.WildcardOne term) throws E { visitWildcardOne(term); return null; } public final Object CaseWildcardAny(jcs.ast.gen.WildcardAny term) throws E { visitWildcardAny(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierList(jcs.ast.gen.IdentifierList term) throws E { visitIdentifierList(term); return null; } public final Object CaseIntersectionType(jcs.ast.gen.IntersectionType term) throws E { visitIntersectionType(term); return null; } public final Object CaseBasicType(jcs.ast.gen.BasicType term) throws E { visitBasicType(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitExpression(jcs.ast.gen.Expression term) throws E {
            term.Switch(variantVisit$Expression);
        }

        public final void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E {
            term.Switch(variantVisit$IdentifierOpt);
        }

        public final void visitStatement(jcs.ast.gen.Statement term) throws E {
            term.Switch(variantVisit$Statement);
        }

        public final void visitType(jcs.ast.gen.Type term) throws E {
            term.Switch(variantVisit$Type);
        }
    }

    static class Impl extends KatjaListImpl<jcs.ast.gen.Statement> implements jcs.ast.gen.Block {

        //----- methods of Impl -----

        Impl(jcs.ast.gen.Statement... elements) {
            super(elements);

            for(jcs.ast.gen.Statement element : elements)
                if(element == null)
                    throw new IllegalArgumentException("constructor of sort Block invoked with null element");
        }

        private Impl() {
        }

        protected jcs.ast.gen.Block createInstance(jcs.ast.gen.Statement[] elements, boolean isSet) {
            for(jcs.ast.gen.Statement element : elements)
                if(element == null)
                    throw new IllegalArgumentException("constructor of sort Block invoked with null element");

            Block.Impl temp = new jcs.ast.gen.Block.Impl();
            temp.values = elements;
            temp = (Block.Impl) JCS.unique(temp);
            if(isSet) temp.set = temp;

            return temp;
        }

        protected jcs.ast.gen.Statement[] createArray(int size) {
            return new jcs.ast.gen.Statement[size];
        }

        public jcs.ast.gen.Block add(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.add(element);
        }

        public jcs.ast.gen.Block addAll(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.addAll(list);
        }

        public jcs.ast.gen.Block remove(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.remove(element);
        }

        public jcs.ast.gen.Block removeAll(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.removeAll(list);
        }

        public jcs.ast.gen.Block appBack(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.appBack(element);
        }

        public jcs.ast.gen.Block appFront(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.appFront(element);
        }

        public jcs.ast.gen.Block conc(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.conc(list);
        }

        public jcs.ast.gen.Block front() {
            return (jcs.ast.gen.Block) super.front();
        }

        public jcs.ast.gen.Block back() {
            return (jcs.ast.gen.Block) super.back();
        }

        public jcs.ast.gen.Block reverse() {
            return (jcs.ast.gen.Block) super.reverse();
        }

        public jcs.ast.gen.Block sublist(int from, int to) {
            return (jcs.ast.gen.Block) super.sublist(from, to);
        }

        public jcs.ast.gen.Block replace(int ith, Object element) {
            return (jcs.ast.gen.Block) super.replace(ith, element);
        }

        public jcs.ast.gen.Block toSet() {
            return (jcs.ast.gen.Block) super.toSet();
        }

        public jcs.ast.gen.Block setAdd(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.setAdd(element);
        }

        public jcs.ast.gen.Block setRemove(jcs.ast.gen.Statement element) {
            return (jcs.ast.gen.Block) super.setRemove(element);
        }

        public jcs.ast.gen.Block setUnion(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.setUnion(list);
        }

        public jcs.ast.gen.Block setIntersection(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.setIntersection(list);
        }

        public jcs.ast.gen.Block setWithout(KatjaList<? extends jcs.ast.gen.Statement> list) {
            return (jcs.ast.gen.Block) super.setWithout(list);
        }

        public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseBlock(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            boolean first = true;

            builder.append("JCS.Block");
            builder.append("( ");
            for(jcs.ast.gen.Statement element : values) {
                if(first) first = false;
                else builder.append(", ");
                element.toJavaCode(builder);
            }
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            boolean first = true;

            builder.append("Block");
            builder.append("( ");
            for(jcs.ast.gen.Statement element : values) {
                if(first) first = false;
                else builder.append(", ");
                element.toString(builder);
            }
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "Block";
        }
    }
}

