package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface Call extends jcs.ast.gen.Statement, KatjaTuple {

    //----- methods of Call -----

    public jcs.ast.gen.Type type();
    public jcs.ast.gen.Call replaceType(jcs.ast.gen.Type type);
    public jcs.ast.gen.IdentifierOpt identCallee();
    public jcs.ast.gen.Call replaceIdentCallee(jcs.ast.gen.IdentifierOpt identCallee);
    public jcs.ast.gen.Identifier methodName();
    public jcs.ast.gen.Call replaceMethodName(jcs.ast.gen.Identifier methodName);
    public jcs.ast.gen.ParameterList params();
    public jcs.ast.gen.Call replaceParams(jcs.ast.gen.ParameterList params);
    public jcs.ast.gen.IdentifierOpt identReturn();
    public jcs.ast.gen.Call replaceIdentReturn(jcs.ast.gen.IdentifierOpt identReturn);
    public KatjaSort get(int i);
    public int size();
    public jcs.ast.gen.Call replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E;

    //----- nested classes of Call -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitType(jcs.ast.gen.Type term) throws E;
        public void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitParameterList(jcs.ast.gen.ParameterList term) throws E;
        public void visitWildcardOne(jcs.ast.gen.WildcardOne term) throws E;
        public void visitWildcardAny(jcs.ast.gen.WildcardAny term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitIntersectionType(jcs.ast.gen.IntersectionType term) throws E;
        public void visitBasicType(jcs.ast.gen.BasicType term) throws E;
        public void visitIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitParameter(jcs.ast.gen.Parameter term) throws E;
        public void visitCall(jcs.ast.gen.Call term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.Call.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.IdentifierOpt.Switch<Object, E> variantVisit$IdentifierOpt = new jcs.ast.gen.IdentifierOpt.Switch<Object, E>() { public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E { visitIdentifierNone(term); return null; } };
        private final jcs.ast.gen.Type.Switch<Object, E> variantVisit$Type = new jcs.ast.gen.Type.Switch<Object, E>() { public final Object CaseWildcardOne(jcs.ast.gen.WildcardOne term) throws E { visitWildcardOne(term); return null; } public final Object CaseWildcardAny(jcs.ast.gen.WildcardAny term) throws E { visitWildcardAny(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierList(jcs.ast.gen.IdentifierList term) throws E { visitIdentifierList(term); return null; } public final Object CaseIntersectionType(jcs.ast.gen.IntersectionType term) throws E { visitIntersectionType(term); return null; } public final Object CaseBasicType(jcs.ast.gen.BasicType term) throws E { visitBasicType(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E {
            term.Switch(variantVisit$IdentifierOpt);
        }

        public final void visitType(jcs.ast.gen.Type term) throws E {
            term.Switch(variantVisit$Type);
        }
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.Call {

        //----- attributes of Impl -----

        private jcs.ast.gen.Type _type = null;
        private jcs.ast.gen.IdentifierOpt _identCallee = null;
        private jcs.ast.gen.Identifier _methodName = null;
        private jcs.ast.gen.ParameterList _params = null;
        private jcs.ast.gen.IdentifierOpt _identReturn = null;

        //----- methods of Impl -----

        public jcs.ast.gen.Type type() {
            return _type;
        }

        public jcs.ast.gen.Call replaceType(jcs.ast.gen.Type type) {
            return replace(0, type);
        }

        public jcs.ast.gen.IdentifierOpt identCallee() {
            return _identCallee;
        }

        public jcs.ast.gen.Call replaceIdentCallee(jcs.ast.gen.IdentifierOpt identCallee) {
            return replace(1, identCallee);
        }

        public jcs.ast.gen.Identifier methodName() {
            return _methodName;
        }

        public jcs.ast.gen.Call replaceMethodName(jcs.ast.gen.Identifier methodName) {
            return replace(2, methodName);
        }

        public jcs.ast.gen.ParameterList params() {
            return _params;
        }

        public jcs.ast.gen.Call replaceParams(jcs.ast.gen.ParameterList params) {
            return replace(3, params);
        }

        public jcs.ast.gen.IdentifierOpt identReturn() {
            return _identReturn;
        }

        public jcs.ast.gen.Call replaceIdentReturn(jcs.ast.gen.IdentifierOpt identReturn) {
            return replace(4, identReturn);
        }

        Impl(jcs.ast.gen.Type type, jcs.ast.gen.IdentifierOpt identCallee, jcs.ast.gen.Identifier methodName, jcs.ast.gen.ParameterList params, jcs.ast.gen.IdentifierOpt identReturn) {
            if(type == null)
                throw new IllegalArgumentException("constructor of sort Call invoked with null parameter type");
            if(identCallee == null)
                throw new IllegalArgumentException("constructor of sort Call invoked with null parameter identCallee");
            if(methodName == null)
                throw new IllegalArgumentException("constructor of sort Call invoked with null parameter methodName");
            if(params == null)
                throw new IllegalArgumentException("constructor of sort Call invoked with null parameter params");
            if(identReturn == null)
                throw new IllegalArgumentException("constructor of sort Call invoked with null parameter identReturn");

            this._type = type;
            this._identCallee = identCallee;
            this._methodName = methodName;
            this._params = params;
            this._identReturn = identReturn;
        }

        public KatjaSort get(int i) {
            int ith = i;

            if(ith < 0) ith += 5;

            switch(ith) {
                case 0: return _type;
                case 1: return _identCallee;
                case 2: return _methodName;
                case 3: return _params;
                case 4: return _identReturn;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort Call invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort Call invoked with too large parameter "+i+", sort has only 5 components");
                    }
            }
        }

        public int size() {
            return 5;
        }

        public jcs.ast.gen.Call replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort Call invoked with negative parameter "+pos);
            if(pos >= 5)
                throw new IllegalArgumentException("replace on sort Call invoked with too large parameter "+pos+", sort has only 5 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.Type))
                throw new IllegalArgumentException("replace on sort Call invoked with term of incorrect sort, Type expected");
            if(pos == 1 && !(term instanceof jcs.ast.gen.IdentifierOpt))
                throw new IllegalArgumentException("replace on sort Call invoked with term of incorrect sort, IdentifierOpt expected");
            if(pos == 2 && !(term instanceof jcs.ast.gen.Identifier))
                throw new IllegalArgumentException("replace on sort Call invoked with term of incorrect sort, Identifier expected");
            if(pos == 3 && !(term instanceof jcs.ast.gen.ParameterList))
                throw new IllegalArgumentException("replace on sort Call invoked with term of incorrect sort, ParameterList expected");
            if(pos == 4 && !(term instanceof jcs.ast.gen.IdentifierOpt))
                throw new IllegalArgumentException("replace on sort Call invoked with term of incorrect sort, IdentifierOpt expected");

            return (jcs.ast.gen.Call) JCS.unique(new jcs.ast.gen.Call.Impl(
                pos == 0 ? (jcs.ast.gen.Type) term : _type,
                pos == 1 ? (jcs.ast.gen.IdentifierOpt) term : _identCallee,
                pos == 2 ? (jcs.ast.gen.Identifier) term : _methodName,
                pos == 3 ? (jcs.ast.gen.ParameterList) term : _params,
                pos == 4 ? (jcs.ast.gen.IdentifierOpt) term : _identReturn
            ));
        }

        public <CT, E extends Throwable> CT Switch(jcs.ast.gen.Statement.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseCall(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.Call");
            builder.append("( ");
            _type.toJavaCode(builder);
            builder.append(", ");
            _identCallee.toJavaCode(builder);
            builder.append(", ");
            _methodName.toJavaCode(builder);
            builder.append(", ");
            _params.toJavaCode(builder);
            builder.append(", ");
            _identReturn.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("Call");
            builder.append("( ");
            _type.toString(builder);
            builder.append(", ");
            _identCallee.toString(builder);
            builder.append(", ");
            _methodName.toString(builder);
            builder.append(", ");
            _params.toString(builder);
            builder.append(", ");
            _identReturn.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "Call";
        }
    }
}

