package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface MatchDeclaration extends KatjaTuple {

    //----- methods of MatchDeclaration -----

    public jcs.ast.gen.TypeList types();
    public jcs.ast.gen.MatchDeclaration replaceTypes(jcs.ast.gen.TypeList types);
    public jcs.ast.gen.MethodDeclarationList methods();
    public jcs.ast.gen.MatchDeclaration replaceMethods(jcs.ast.gen.MethodDeclarationList methods);
    public KatjaList get(int i);
    public int size();
    public jcs.ast.gen.MatchDeclaration replace(int pos, Object term);

    //----- nested classes of MatchDeclaration -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitTypeList(jcs.ast.gen.TypeList term) throws E;
        public void visitMethodDeclarationList(jcs.ast.gen.MethodDeclarationList term) throws E;
        public void visitType(jcs.ast.gen.Type term) throws E;
        public void visitMethodDeclaration(jcs.ast.gen.MethodDeclaration term) throws E;
        public void visitWildcardOne(jcs.ast.gen.WildcardOne term) throws E;
        public void visitWildcardAny(jcs.ast.gen.WildcardAny term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitIntersectionType(jcs.ast.gen.IntersectionType term) throws E;
        public void visitBasicType(jcs.ast.gen.BasicType term) throws E;
        public void visitParameterList(jcs.ast.gen.ParameterList term) throws E;
        public void visitBlock(jcs.ast.gen.Block term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitParameter(jcs.ast.gen.Parameter term) throws E;
        public void visitStatement(jcs.ast.gen.Statement term) throws E;
        public void visitOr(jcs.ast.gen.Or term) throws E;
        public void visitLoop(jcs.ast.gen.Loop term) throws E;
        public void visitAssume(jcs.ast.gen.Assume term) throws E;
        public void visitAssert(jcs.ast.gen.Assert term) throws E;
        public void visitCall(jcs.ast.gen.Call term) throws E;
        public void visitEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E;
        public void visitBreak(jcs.ast.gen.Break term) throws E;
        public void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E;
        public void visitExpression(jcs.ast.gen.Expression term) throws E;
        public void visitIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E;
        public void visitOrExpression(jcs.ast.gen.OrExpression term) throws E;
        public void visitAndExpression(jcs.ast.gen.AndExpression term) throws E;
        public void visitEqExpression(jcs.ast.gen.EqExpression term) throws E;
        public void visitNotExpression(jcs.ast.gen.NotExpression term) throws E;
        public void visitLiteral(jcs.ast.gen.Literal term) throws E;
        public void visitBoolValue(jcs.ast.gen.BoolValue term) throws E;
        public void visitIntValue(jcs.ast.gen.IntValue term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitMatchDeclaration(jcs.ast.gen.MatchDeclaration term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.MatchDeclaration.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.Expression.Switch<Object, E> variantVisit$Expression = new jcs.ast.gen.Expression.Switch<Object, E>() { public final Object CaseOrExpression(jcs.ast.gen.OrExpression term) throws E { visitOrExpression(term); return null; } public final Object CaseAndExpression(jcs.ast.gen.AndExpression term) throws E { visitAndExpression(term); return null; } public final Object CaseEqExpression(jcs.ast.gen.EqExpression term) throws E { visitEqExpression(term); return null; } public final Object CaseNotExpression(jcs.ast.gen.NotExpression term) throws E { visitNotExpression(term); return null; } public final Object CaseLiteral(jcs.ast.gen.Literal term) throws E { visitLiteral(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseBoolValue(jcs.ast.gen.BoolValue term) throws E { visitBoolValue(term); return null; } public final Object CaseIntValue(jcs.ast.gen.IntValue term) throws E { visitIntValue(term); return null; } };
        private final jcs.ast.gen.IdentifierOpt.Switch<Object, E> variantVisit$IdentifierOpt = new jcs.ast.gen.IdentifierOpt.Switch<Object, E>() { public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E { visitIdentifierNone(term); return null; } };
        private final jcs.ast.gen.Statement.Switch<Object, E> variantVisit$Statement = new jcs.ast.gen.Statement.Switch<Object, E>() { public final Object CaseOr(jcs.ast.gen.Or term) throws E { visitOr(term); return null; } public final Object CaseLoop(jcs.ast.gen.Loop term) throws E { visitLoop(term); return null; } public final Object CaseBlock(jcs.ast.gen.Block term) throws E { visitBlock(term); return null; } public final Object CaseAssume(jcs.ast.gen.Assume term) throws E { visitAssume(term); return null; } public final Object CaseAssert(jcs.ast.gen.Assert term) throws E { visitAssert(term); return null; } public final Object CaseCall(jcs.ast.gen.Call term) throws E { visitCall(term); return null; } public final Object CaseEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E { visitEmptyStatement(term); return null; } public final Object CaseBreak(jcs.ast.gen.Break term) throws E { visitBreak(term); return null; } };
        private final jcs.ast.gen.Type.Switch<Object, E> variantVisit$Type = new jcs.ast.gen.Type.Switch<Object, E>() { public final Object CaseWildcardOne(jcs.ast.gen.WildcardOne term) throws E { visitWildcardOne(term); return null; } public final Object CaseWildcardAny(jcs.ast.gen.WildcardAny term) throws E { visitWildcardAny(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierList(jcs.ast.gen.IdentifierList term) throws E { visitIdentifierList(term); return null; } public final Object CaseIntersectionType(jcs.ast.gen.IntersectionType term) throws E { visitIntersectionType(term); return null; } public final Object CaseBasicType(jcs.ast.gen.BasicType term) throws E { visitBasicType(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitExpression(jcs.ast.gen.Expression term) throws E {
            term.Switch(variantVisit$Expression);
        }

        public final void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E {
            term.Switch(variantVisit$IdentifierOpt);
        }

        public final void visitStatement(jcs.ast.gen.Statement term) throws E {
            term.Switch(variantVisit$Statement);
        }

        public final void visitType(jcs.ast.gen.Type term) throws E {
            term.Switch(variantVisit$Type);
        }
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.MatchDeclaration {

        //----- attributes of Impl -----

        private jcs.ast.gen.TypeList _types = null;
        private jcs.ast.gen.MethodDeclarationList _methods = null;

        //----- methods of Impl -----

        public jcs.ast.gen.TypeList types() {
            return _types;
        }

        public jcs.ast.gen.MatchDeclaration replaceTypes(jcs.ast.gen.TypeList types) {
            return replace(0, types);
        }

        public jcs.ast.gen.MethodDeclarationList methods() {
            return _methods;
        }

        public jcs.ast.gen.MatchDeclaration replaceMethods(jcs.ast.gen.MethodDeclarationList methods) {
            return replace(1, methods);
        }

        Impl(jcs.ast.gen.TypeList types, jcs.ast.gen.MethodDeclarationList methods) {
            if(types == null)
                throw new IllegalArgumentException("constructor of sort MatchDeclaration invoked with null parameter types");
            if(methods == null)
                throw new IllegalArgumentException("constructor of sort MatchDeclaration invoked with null parameter methods");

            this._types = types;
            this._methods = methods;
        }

        public KatjaList get(int i) {
            int ith = i;

            if(ith < 0) ith += 2;

            switch(ith) {
                case 0: return _types;
                case 1: return _methods;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort MatchDeclaration invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort MatchDeclaration invoked with too large parameter "+i+", sort has only 2 components");
                    }
            }
        }

        public int size() {
            return 2;
        }

        public jcs.ast.gen.MatchDeclaration replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort MatchDeclaration invoked with negative parameter "+pos);
            if(pos >= 2)
                throw new IllegalArgumentException("replace on sort MatchDeclaration invoked with too large parameter "+pos+", sort has only 2 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.TypeList))
                throw new IllegalArgumentException("replace on sort MatchDeclaration invoked with term of incorrect sort, TypeList expected");
            if(pos == 1 && !(term instanceof jcs.ast.gen.MethodDeclarationList))
                throw new IllegalArgumentException("replace on sort MatchDeclaration invoked with term of incorrect sort, MethodDeclarationList expected");

            return (jcs.ast.gen.MatchDeclaration) JCS.unique(new jcs.ast.gen.MatchDeclaration.Impl(
                pos == 0 ? (jcs.ast.gen.TypeList) term : _types,
                pos == 1 ? (jcs.ast.gen.MethodDeclarationList) term : _methods
            ));
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.MatchDeclaration");
            builder.append("( ");
            _types.toJavaCode(builder);
            builder.append(", ");
            _methods.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("MatchDeclaration");
            builder.append("( ");
            _types.toString(builder);
            builder.append(", ");
            _methods.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "MatchDeclaration";
        }
    }
}

