package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface MethodDeclaration extends KatjaTuple {

    //----- methods of MethodDeclaration -----

    public jcs.ast.gen.Type type();
    public jcs.ast.gen.MethodDeclaration replaceType(jcs.ast.gen.Type type);
    public jcs.ast.gen.Identifier id();
    public jcs.ast.gen.MethodDeclaration replaceId(jcs.ast.gen.Identifier id);
    public jcs.ast.gen.ParameterList params();
    public jcs.ast.gen.MethodDeclaration replaceParams(jcs.ast.gen.ParameterList params);
    public jcs.ast.gen.Block block();
    public jcs.ast.gen.MethodDeclaration replaceBlock(jcs.ast.gen.Block block);
    public KatjaSort get(int i);
    public int size();
    public jcs.ast.gen.MethodDeclaration replace(int pos, Object term);

    //----- nested classes of MethodDeclaration -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitType(jcs.ast.gen.Type term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitParameterList(jcs.ast.gen.ParameterList term) throws E;
        public void visitBlock(jcs.ast.gen.Block term) throws E;
        public void visitWildcardOne(jcs.ast.gen.WildcardOne term) throws E;
        public void visitWildcardAny(jcs.ast.gen.WildcardAny term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitIntersectionType(jcs.ast.gen.IntersectionType term) throws E;
        public void visitBasicType(jcs.ast.gen.BasicType term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitParameter(jcs.ast.gen.Parameter term) throws E;
        public void visitStatement(jcs.ast.gen.Statement term) throws E;
        public void visitOr(jcs.ast.gen.Or term) throws E;
        public void visitLoop(jcs.ast.gen.Loop term) throws E;
        public void visitAssume(jcs.ast.gen.Assume term) throws E;
        public void visitAssert(jcs.ast.gen.Assert term) throws E;
        public void visitCall(jcs.ast.gen.Call term) throws E;
        public void visitEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E;
        public void visitBreak(jcs.ast.gen.Break term) throws E;
        public void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E;
        public void visitExpression(jcs.ast.gen.Expression term) throws E;
        public void visitIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E;
        public void visitOrExpression(jcs.ast.gen.OrExpression term) throws E;
        public void visitAndExpression(jcs.ast.gen.AndExpression term) throws E;
        public void visitEqExpression(jcs.ast.gen.EqExpression term) throws E;
        public void visitNotExpression(jcs.ast.gen.NotExpression term) throws E;
        public void visitLiteral(jcs.ast.gen.Literal term) throws E;
        public void visitBoolValue(jcs.ast.gen.BoolValue term) throws E;
        public void visitIntValue(jcs.ast.gen.IntValue term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitMethodDeclaration(jcs.ast.gen.MethodDeclaration term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.MethodDeclaration.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.Expression.Switch<Object, E> variantVisit$Expression = new jcs.ast.gen.Expression.Switch<Object, E>() { public final Object CaseOrExpression(jcs.ast.gen.OrExpression term) throws E { visitOrExpression(term); return null; } public final Object CaseAndExpression(jcs.ast.gen.AndExpression term) throws E { visitAndExpression(term); return null; } public final Object CaseEqExpression(jcs.ast.gen.EqExpression term) throws E { visitEqExpression(term); return null; } public final Object CaseNotExpression(jcs.ast.gen.NotExpression term) throws E { visitNotExpression(term); return null; } public final Object CaseLiteral(jcs.ast.gen.Literal term) throws E { visitLiteral(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseBoolValue(jcs.ast.gen.BoolValue term) throws E { visitBoolValue(term); return null; } public final Object CaseIntValue(jcs.ast.gen.IntValue term) throws E { visitIntValue(term); return null; } };
        private final jcs.ast.gen.IdentifierOpt.Switch<Object, E> variantVisit$IdentifierOpt = new jcs.ast.gen.IdentifierOpt.Switch<Object, E>() { public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E { visitIdentifierNone(term); return null; } };
        private final jcs.ast.gen.Statement.Switch<Object, E> variantVisit$Statement = new jcs.ast.gen.Statement.Switch<Object, E>() { public final Object CaseOr(jcs.ast.gen.Or term) throws E { visitOr(term); return null; } public final Object CaseLoop(jcs.ast.gen.Loop term) throws E { visitLoop(term); return null; } public final Object CaseBlock(jcs.ast.gen.Block term) throws E { visitBlock(term); return null; } public final Object CaseAssume(jcs.ast.gen.Assume term) throws E { visitAssume(term); return null; } public final Object CaseAssert(jcs.ast.gen.Assert term) throws E { visitAssert(term); return null; } public final Object CaseCall(jcs.ast.gen.Call term) throws E { visitCall(term); return null; } public final Object CaseEmptyStatement(jcs.ast.gen.EmptyStatement term) throws E { visitEmptyStatement(term); return null; } public final Object CaseBreak(jcs.ast.gen.Break term) throws E { visitBreak(term); return null; } };
        private final jcs.ast.gen.Type.Switch<Object, E> variantVisit$Type = new jcs.ast.gen.Type.Switch<Object, E>() { public final Object CaseWildcardOne(jcs.ast.gen.WildcardOne term) throws E { visitWildcardOne(term); return null; } public final Object CaseWildcardAny(jcs.ast.gen.WildcardAny term) throws E { visitWildcardAny(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierList(jcs.ast.gen.IdentifierList term) throws E { visitIdentifierList(term); return null; } public final Object CaseIntersectionType(jcs.ast.gen.IntersectionType term) throws E { visitIntersectionType(term); return null; } public final Object CaseBasicType(jcs.ast.gen.BasicType term) throws E { visitBasicType(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitExpression(jcs.ast.gen.Expression term) throws E {
            term.Switch(variantVisit$Expression);
        }

        public final void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E {
            term.Switch(variantVisit$IdentifierOpt);
        }

        public final void visitStatement(jcs.ast.gen.Statement term) throws E {
            term.Switch(variantVisit$Statement);
        }

        public final void visitType(jcs.ast.gen.Type term) throws E {
            term.Switch(variantVisit$Type);
        }
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.MethodDeclaration {

        //----- attributes of Impl -----

        private jcs.ast.gen.Type _type = null;
        private jcs.ast.gen.Identifier _id = null;
        private jcs.ast.gen.ParameterList _params = null;
        private jcs.ast.gen.Block _block = null;

        //----- methods of Impl -----

        public jcs.ast.gen.Type type() {
            return _type;
        }

        public jcs.ast.gen.MethodDeclaration replaceType(jcs.ast.gen.Type type) {
            return replace(0, type);
        }

        public jcs.ast.gen.Identifier id() {
            return _id;
        }

        public jcs.ast.gen.MethodDeclaration replaceId(jcs.ast.gen.Identifier id) {
            return replace(1, id);
        }

        public jcs.ast.gen.ParameterList params() {
            return _params;
        }

        public jcs.ast.gen.MethodDeclaration replaceParams(jcs.ast.gen.ParameterList params) {
            return replace(2, params);
        }

        public jcs.ast.gen.Block block() {
            return _block;
        }

        public jcs.ast.gen.MethodDeclaration replaceBlock(jcs.ast.gen.Block block) {
            return replace(3, block);
        }

        Impl(jcs.ast.gen.Type type, jcs.ast.gen.Identifier id, jcs.ast.gen.ParameterList params, jcs.ast.gen.Block block) {
            if(type == null)
                throw new IllegalArgumentException("constructor of sort MethodDeclaration invoked with null parameter type");
            if(id == null)
                throw new IllegalArgumentException("constructor of sort MethodDeclaration invoked with null parameter id");
            if(params == null)
                throw new IllegalArgumentException("constructor of sort MethodDeclaration invoked with null parameter params");
            if(block == null)
                throw new IllegalArgumentException("constructor of sort MethodDeclaration invoked with null parameter block");

            this._type = type;
            this._id = id;
            this._params = params;
            this._block = block;
        }

        public KatjaSort get(int i) {
            int ith = i;

            if(ith < 0) ith += 4;

            switch(ith) {
                case 0: return _type;
                case 1: return _id;
                case 2: return _params;
                case 3: return _block;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort MethodDeclaration invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort MethodDeclaration invoked with too large parameter "+i+", sort has only 4 components");
                    }
            }
        }

        public int size() {
            return 4;
        }

        public jcs.ast.gen.MethodDeclaration replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with negative parameter "+pos);
            if(pos >= 4)
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with too large parameter "+pos+", sort has only 4 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.Type))
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with term of incorrect sort, Type expected");
            if(pos == 1 && !(term instanceof jcs.ast.gen.Identifier))
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with term of incorrect sort, Identifier expected");
            if(pos == 2 && !(term instanceof jcs.ast.gen.ParameterList))
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with term of incorrect sort, ParameterList expected");
            if(pos == 3 && !(term instanceof jcs.ast.gen.Block))
                throw new IllegalArgumentException("replace on sort MethodDeclaration invoked with term of incorrect sort, Block expected");

            return (jcs.ast.gen.MethodDeclaration) JCS.unique(new jcs.ast.gen.MethodDeclaration.Impl(
                pos == 0 ? (jcs.ast.gen.Type) term : _type,
                pos == 1 ? (jcs.ast.gen.Identifier) term : _id,
                pos == 2 ? (jcs.ast.gen.ParameterList) term : _params,
                pos == 3 ? (jcs.ast.gen.Block) term : _block
            ));
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.MethodDeclaration");
            builder.append("( ");
            _type.toJavaCode(builder);
            builder.append(", ");
            _id.toJavaCode(builder);
            builder.append(", ");
            _params.toJavaCode(builder);
            builder.append(", ");
            _block.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("MethodDeclaration");
            builder.append("( ");
            _type.toString(builder);
            builder.append(", ");
            _id.toString(builder);
            builder.append(", ");
            _params.toString(builder);
            builder.append(", ");
            _block.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "MethodDeclaration";
        }
    }
}

