package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface Parameter extends KatjaTuple {

    //----- methods of Parameter -----

    public jcs.ast.gen.Type type();
    public jcs.ast.gen.Parameter replaceType(jcs.ast.gen.Type type);
    public jcs.ast.gen.IdentifierOpt ident();
    public jcs.ast.gen.Parameter replaceIdent(jcs.ast.gen.IdentifierOpt ident);
    public KatjaSort get(int i);
    public int size();
    public jcs.ast.gen.Parameter replace(int pos, Object term);

    //----- nested classes of Parameter -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitType(jcs.ast.gen.Type term) throws E;
        public void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E;
        public void visitWildcardOne(jcs.ast.gen.WildcardOne term) throws E;
        public void visitWildcardAny(jcs.ast.gen.WildcardAny term) throws E;
        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitIntersectionType(jcs.ast.gen.IntersectionType term) throws E;
        public void visitBasicType(jcs.ast.gen.BasicType term) throws E;
        public void visitIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitParameter(jcs.ast.gen.Parameter term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.Parameter.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final jcs.ast.gen.IdentifierOpt.Switch<Object, E> variantVisit$IdentifierOpt = new jcs.ast.gen.IdentifierOpt.Switch<Object, E>() { public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierNone(jcs.ast.gen.IdentifierNone term) throws E { visitIdentifierNone(term); return null; } };
        private final jcs.ast.gen.Type.Switch<Object, E> variantVisit$Type = new jcs.ast.gen.Type.Switch<Object, E>() { public final Object CaseWildcardOne(jcs.ast.gen.WildcardOne term) throws E { visitWildcardOne(term); return null; } public final Object CaseWildcardAny(jcs.ast.gen.WildcardAny term) throws E { visitWildcardAny(term); return null; } public final Object CaseIdentifier(jcs.ast.gen.Identifier term) throws E { visitIdentifier(term); return null; } public final Object CaseIdentifierList(jcs.ast.gen.IdentifierList term) throws E { visitIdentifierList(term); return null; } public final Object CaseIntersectionType(jcs.ast.gen.IntersectionType term) throws E { visitIntersectionType(term); return null; } public final Object CaseBasicType(jcs.ast.gen.BasicType term) throws E { visitBasicType(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitIdentifierOpt(jcs.ast.gen.IdentifierOpt term) throws E {
            term.Switch(variantVisit$IdentifierOpt);
        }

        public final void visitType(jcs.ast.gen.Type term) throws E {
            term.Switch(variantVisit$Type);
        }
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.Parameter {

        //----- attributes of Impl -----

        private jcs.ast.gen.Type _type = null;
        private jcs.ast.gen.IdentifierOpt _ident = null;

        //----- methods of Impl -----

        public jcs.ast.gen.Type type() {
            return _type;
        }

        public jcs.ast.gen.Parameter replaceType(jcs.ast.gen.Type type) {
            return replace(0, type);
        }

        public jcs.ast.gen.IdentifierOpt ident() {
            return _ident;
        }

        public jcs.ast.gen.Parameter replaceIdent(jcs.ast.gen.IdentifierOpt ident) {
            return replace(1, ident);
        }

        Impl(jcs.ast.gen.Type type, jcs.ast.gen.IdentifierOpt ident) {
            if(type == null)
                throw new IllegalArgumentException("constructor of sort Parameter invoked with null parameter type");
            if(ident == null)
                throw new IllegalArgumentException("constructor of sort Parameter invoked with null parameter ident");

            this._type = type;
            this._ident = ident;
        }

        public KatjaSort get(int i) {
            int ith = i;

            if(ith < 0) ith += 2;

            switch(ith) {
                case 0: return _type;
                case 1: return _ident;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort Parameter invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort Parameter invoked with too large parameter "+i+", sort has only 2 components");
                    }
            }
        }

        public int size() {
            return 2;
        }

        public jcs.ast.gen.Parameter replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort Parameter invoked with negative parameter "+pos);
            if(pos >= 2)
                throw new IllegalArgumentException("replace on sort Parameter invoked with too large parameter "+pos+", sort has only 2 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.Type))
                throw new IllegalArgumentException("replace on sort Parameter invoked with term of incorrect sort, Type expected");
            if(pos == 1 && !(term instanceof jcs.ast.gen.IdentifierOpt))
                throw new IllegalArgumentException("replace on sort Parameter invoked with term of incorrect sort, IdentifierOpt expected");

            return (jcs.ast.gen.Parameter) JCS.unique(new jcs.ast.gen.Parameter.Impl(
                pos == 0 ? (jcs.ast.gen.Type) term : _type,
                pos == 1 ? (jcs.ast.gen.IdentifierOpt) term : _ident
            ));
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.Parameter");
            builder.append("( ");
            _type.toJavaCode(builder);
            builder.append(", ");
            _ident.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("Parameter");
            builder.append("( ");
            _type.toString(builder);
            builder.append(", ");
            _ident.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "Parameter";
        }
    }
}

