package jcs.ast.gen;

import katja.common.*;
import java.io.IOException;

public interface StateDeclaration extends KatjaTuple {

    //----- methods of StateDeclaration -----

    public jcs.ast.gen.Identifier ident();
    public jcs.ast.gen.StateDeclaration replaceIdent(jcs.ast.gen.Identifier ident);
    public jcs.ast.gen.IdentifierList idents();
    public jcs.ast.gen.StateDeclaration replaceIdents(jcs.ast.gen.IdentifierList idents);
    public jcs.ast.gen.Type get(int i);
    public int size();
    public jcs.ast.gen.StateDeclaration replace(int pos, Object term);

    //----- nested classes of StateDeclaration -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitIdentifier(jcs.ast.gen.Identifier term) throws E;
        public void visitIdentifierList(jcs.ast.gen.IdentifierList term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitStateDeclaration(jcs.ast.gen.StateDeclaration term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements jcs.ast.gen.StateDeclaration.VisitorType<E> {
    }

    static class Impl extends KatjaTupleImpl implements jcs.ast.gen.StateDeclaration {

        //----- attributes of Impl -----

        private jcs.ast.gen.Identifier _ident = null;
        private jcs.ast.gen.IdentifierList _idents = null;

        //----- methods of Impl -----

        public jcs.ast.gen.Identifier ident() {
            return _ident;
        }

        public jcs.ast.gen.StateDeclaration replaceIdent(jcs.ast.gen.Identifier ident) {
            return replace(0, ident);
        }

        public jcs.ast.gen.IdentifierList idents() {
            return _idents;
        }

        public jcs.ast.gen.StateDeclaration replaceIdents(jcs.ast.gen.IdentifierList idents) {
            return replace(1, idents);
        }

        Impl(jcs.ast.gen.Identifier ident, jcs.ast.gen.IdentifierList idents) {
            if(ident == null)
                throw new IllegalArgumentException("constructor of sort StateDeclaration invoked with null parameter ident");
            if(idents == null)
                throw new IllegalArgumentException("constructor of sort StateDeclaration invoked with null parameter idents");

            this._ident = ident;
            this._idents = idents;
        }

        public jcs.ast.gen.Type get(int i) {
            int ith = i;

            if(ith < 0) ith += 2;

            switch(ith) {
                case 0: return _ident;
                case 1: return _idents;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort StateDeclaration invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort StateDeclaration invoked with too large parameter "+i+", sort has only 2 components");
                    }
            }
        }

        public int size() {
            return 2;
        }

        public jcs.ast.gen.StateDeclaration replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort StateDeclaration invoked with negative parameter "+pos);
            if(pos >= 2)
                throw new IllegalArgumentException("replace on sort StateDeclaration invoked with too large parameter "+pos+", sort has only 2 components");

            if(pos == 0 && !(term instanceof jcs.ast.gen.Identifier))
                throw new IllegalArgumentException("replace on sort StateDeclaration invoked with term of incorrect sort, Identifier expected");
            if(pos == 1 && !(term instanceof jcs.ast.gen.IdentifierList))
                throw new IllegalArgumentException("replace on sort StateDeclaration invoked with term of incorrect sort, IdentifierList expected");

            return (jcs.ast.gen.StateDeclaration) JCS.unique(new jcs.ast.gen.StateDeclaration.Impl(
                pos == 0 ? (jcs.ast.gen.Identifier) term : _ident,
                pos == 1 ? (jcs.ast.gen.IdentifierList) term : _idents
            ));
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("JCS.StateDeclaration");
            builder.append("( ");
            _ident.toJavaCode(builder);
            builder.append(", ");
            _idents.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("StateDeclaration");
            builder.append("( ");
            _ident.toString(builder);
            builder.append(", ");
            _idents.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "StateDeclaration";
        }
    }
}

