/*
    Katja
    Copyright (C) 2003-2007 see README file for authors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package katja.common;

import katja.common.KatjaLeafPos;

/**
 * User: paddy
 * Date: Jul 16, 2004
 * Time: 1:02:54 PM
 */
public abstract class KatjaLeafPosImpl<R extends KatjaSortPos<R>, T> extends KatjaTermPosImpl<R, T> implements KatjaLeafPos<R, T> {

    /**
     * Constructor of KatjaLeafPos. The term reference has to be supplied in addition to
     * the super constructor.
     * @param term reference to the corresponding term for this pos
     * @param parent reference to parent pos
     */
    protected KatjaLeafPosImpl(KatjaSortPos<R> parent, T term, int pos) {
        super(parent, term, pos);
    }

    /**
     * The number of children this pos has; Always 0 for object positions.
     * @return the number of children of a leaf pos: 0
     */
    public int size() {
        return 0;
    }

    /**
     * This method is enforced by the KatjaTermPos interface but should not be called for leaf positions.
     * @param ith the number of the non-existing child...
     * @return nothing, as it always will throw an Exception
     */
    public KatjaSortPos<R> get(int ith) {
        throw new IndexOutOfBoundsException("get on leaf position invoked, which will never have any children");
    }

    /**
     * Substitutes the term at the current position by the given term.
     * This operations parameter can not be typed much better than KatjaTerm.
     * The return-type is only guaranteed if not invoked on a root position which is
     * therefore forbidden. The operation returns the new position after substitution
     * @param term the term to put at the current position.
     * @return a new position representing the new term in the same context as before
     */
    public KatjaLeafPosImpl<R, ?> replace(Object term) {
        return (KatjaLeafPosImpl<R, ?>) super.replace(term);
    }
}
