/*
    Katja
    Copyright (C) 2003-2007 see README file for authors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package katja.common;

import katja.common.KatjaTerm;
import katja.common.KatjaNodePos;

/**
 * User: paddy
 * Date: Dec 5, 2004
 * Time: 2:57:17 PM
 */
public abstract class KatjaNodePosImpl<R extends KatjaSortPos<R>, T extends KatjaTerm> extends KatjaTermPosImpl<R, T> implements KatjaNodePos<R, T> {

    /**
     * Delegates constructor to super class
     * @param parent the parent position
     * @param pos the relative position to the parent
     */
    protected KatjaNodePosImpl(KatjaSortPos<R> parent, T term, int pos) {
        super(parent, term, pos);
    }

    /**
     * Constructor for root nodes.
     */
    protected KatjaNodePosImpl(T term) {
        super(null, term, -1);
    }

    /**
     * Substitutes the term at the current position by the given term.
     * This operations parameter can not be typed much better than KatjaTerm.
     * The return-type is only guaranteed if not invoked on a root position which is
     * therefore forbidden. The operation returns the new position after substitution
     * @param term the term to put at the current position.
     * @return a new position representing the new term in the same context as before
     */
    public KatjaNodePos<R, ?> replace(Object term) {
        return (KatjaNodePos<R, ?>) super.replace(term);
    }

    /**
     * Better types the replace method
     * @param term the term to put in place
     * @return types the replace method
     */
    public KatjaNodePos<R, T> replace(T term) {
        return (KatjaNodePos<R, T>) super.replace(term);
    }
}
