/*
    Katja
    Copyright (C) 2003-2007 see README file for authors

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package katja.common;

import katja.common.KatjaElement;

import java.util.List;

/**
 * User: paddy
 * Date: Jan 19, 2006
 * Time: 3:31:41 PM
 */
public interface KatjaSortPos<R extends KatjaSortPos<R>> extends KatjaElement, Comparable<KatjaSortPos> {

    /**
     * Returns the number of children of the current position.
     * @return the number of children of the current position
     */
    public int size();

    /**
     * Returns a child position, specified by an index, starting at 0.
     * @param ith the index of the child position
     * @return a child position
     */
    public KatjaSortPos<R> get(int ith);

    public int position();
    public List<Integer> path();
    public List<Integer> pathFrom(KatjaSortPos<R> start);
    public KatjaSortPos<R> follow(List<Integer> path);

    public Object term();
    public R root();

    public KatjaSortPos<R> parent();
    public KatjaSortPos<R> rsib();
    public KatjaSortPos<R> lsib();

    public KatjaSortPos<R> preOrder();

    /**
     * Find the next position in the position structure in pre-order traversal pretending
     * {@code this} had no children.
     * @return the next position in pre-order traversal {@code this} had no children 
     */
    public KatjaSortPos<R> preOrderSkip();
    public KatjaSortPos<R> postOrder();
    public KatjaSortPos<R> postOrderStart();

    public boolean termEquals(KatjaSortPos<?> pos);

    public KatjaSortPos<R> replace(Object term);
}
