package katja.spec;

import katja.common.*;
import java.io.IOException;

public interface External extends katja.spec.SortSpec, KatjaTuple {

    //----- methods of External -----

    public katja.spec.SortId sortId();
    public katja.spec.External replaceSortId(katja.spec.SortId sortId);
    public katja.spec.SortId get(int i);
    public int size();
    public katja.spec.External replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E;

    //----- nested classes of External -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitSortId(katja.spec.SortId term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitExternal(katja.spec.External term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.External.VisitorType<E> {
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.External {

        //----- attributes of Impl -----

        private katja.spec.SortId _sortId = null;

        //----- methods of Impl -----

        public katja.spec.SortId sortId() {
            return _sortId;
        }

        public katja.spec.External replaceSortId(katja.spec.SortId sortId) {
            return replace(0, sortId);
        }

        Impl(katja.spec.SortId sortId) {
            if(sortId == null)
                throw new IllegalArgumentException("constructor of sort External invoked with null parameter sortId");

            this._sortId = sortId;
        }

        public katja.spec.SortId get(int i) {
            int ith = i;

            if(ith < 0) ith += 1;

            switch(ith) {
                case 0: return _sortId;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort External invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort External invoked with too large parameter "+i+", sort has only 1 components");
                    }
            }
        }

        public int size() {
            return 1;
        }

        public katja.spec.External replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort External invoked with negative parameter "+pos);
            if(pos >= 1)
                throw new IllegalArgumentException("replace on sort External invoked with too large parameter "+pos+", sort has only 1 components");

            if(pos == 0 && !(term instanceof katja.spec.SortId))
                throw new IllegalArgumentException("replace on sort External invoked with term of incorrect sort, SortId expected");

            return (katja.spec.External) Katja.unique(new katja.spec.External.Impl(
                pos == 0 ? (katja.spec.SortId) term : _sortId
            ));
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseExternal(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Katja.External");
            builder.append("( ");
            _sortId.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("External");
            builder.append("( ");
            _sortId.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "External";
        }
    }
}

