package katja.spec;

import katja.common.*;
import java.io.IOException;

public interface ListProd extends katja.spec.Production, KatjaTuple {

    //----- methods of ListProd -----

    public katja.spec.SortId sortId();
    public katja.spec.ListProd replaceSortId(katja.spec.SortId sortId);
    public katja.spec.SortId param();
    public katja.spec.ListProd replaceParam(katja.spec.SortId param);
    public katja.spec.SortId get(int i);
    public int size();
    public katja.spec.ListProd replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(katja.spec.Production.Switch<CT, E> switchClass) throws E;
    public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E;
    public <CT, E extends Throwable> CT Switch(katja.spec.VariantParam.Switch<CT, E> switchClass) throws E;

    //----- nested classes of ListProd -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitSortId(katja.spec.SortId term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitListProd(katja.spec.ListProd term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.ListProd.VisitorType<E> {
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.ListProd {

        //----- attributes of Impl -----

        private katja.spec.SortId _sortId = null;
        private katja.spec.SortId _param = null;

        //----- methods of Impl -----

        public katja.spec.SortId sortId() {
            return _sortId;
        }

        public katja.spec.ListProd replaceSortId(katja.spec.SortId sortId) {
            return replace(0, sortId);
        }

        public katja.spec.SortId param() {
            return _param;
        }

        public katja.spec.ListProd replaceParam(katja.spec.SortId param) {
            return replace(1, param);
        }

        Impl(katja.spec.SortId sortId, katja.spec.SortId param) {
            if(sortId == null)
                throw new IllegalArgumentException("constructor of sort ListProd invoked with null parameter sortId");
            if(param == null)
                throw new IllegalArgumentException("constructor of sort ListProd invoked with null parameter param");

            this._sortId = sortId;
            this._param = param;
        }

        public katja.spec.SortId get(int i) {
            int ith = i;

            if(ith < 0) ith += 2;

            switch(ith) {
                case 0: return _sortId;
                case 1: return _param;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort ListProd invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort ListProd invoked with too large parameter "+i+", sort has only 2 components");
                    }
            }
        }

        public int size() {
            return 2;
        }

        public katja.spec.ListProd replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort ListProd invoked with negative parameter "+pos);
            if(pos >= 2)
                throw new IllegalArgumentException("replace on sort ListProd invoked with too large parameter "+pos+", sort has only 2 components");

            if(pos == 0 && !(term instanceof katja.spec.SortId))
                throw new IllegalArgumentException("replace on sort ListProd invoked with term of incorrect sort, SortId expected");
            if(pos == 1 && !(term instanceof katja.spec.SortId))
                throw new IllegalArgumentException("replace on sort ListProd invoked with term of incorrect sort, SortId expected");

            return (katja.spec.ListProd) Katja.unique(new katja.spec.ListProd.Impl(
                pos == 0 ? (katja.spec.SortId) term : _sortId,
                pos == 1 ? (katja.spec.SortId) term : _param
            ));
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.Production.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseListProd(this);
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseListProd(this);
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.VariantParam.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseListProd(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Katja.ListProd");
            builder.append("( ");
            _sortId.toJavaCode(builder);
            builder.append(", ");
            _param.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("ListProd");
            builder.append("( ");
            _sortId.toString(builder);
            builder.append(", ");
            _param.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "ListProd";
        }
    }
}

