package katja.spec;

import katja.common.*;
import java.io.IOException;

public interface Selector extends katja.spec.SelectorOpt, KatjaTuple {

    //----- methods of Selector -----

    public java.lang.String name();
    public katja.spec.Selector replaceName(java.lang.String name);
    public java.lang.String get(int i);
    public int size();
    public katja.spec.Selector replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(katja.spec.SelectorOpt.Switch<CT, E> switchClass) throws E;

    //----- nested classes of Selector -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitString(java.lang.String term) throws E;
        public void visitSelector(katja.spec.Selector term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.Selector.VisitorType<E> {
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.Selector {

        //----- attributes of Impl -----

        private java.lang.String _name = null;

        //----- methods of Impl -----

        public java.lang.String name() {
            return _name;
        }

        public katja.spec.Selector replaceName(java.lang.String name) {
            return replace(0, name);
        }

        Impl(java.lang.String name) {
            if(name == null)
                throw new IllegalArgumentException("constructor of sort Selector invoked with null parameter name");

            this._name = name;
        }

        public java.lang.String get(int i) {
            int ith = i;

            if(ith < 0) ith += 1;

            switch(ith) {
                case 0: return _name;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort Selector invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort Selector invoked with too large parameter "+i+", sort has only 1 components");
                    }
            }
        }

        public int size() {
            return 1;
        }

        public katja.spec.Selector replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort Selector invoked with negative parameter "+pos);
            if(pos >= 1)
                throw new IllegalArgumentException("replace on sort Selector invoked with too large parameter "+pos+", sort has only 1 components");

            if(pos == 0 && !(term instanceof java.lang.String))
                throw new IllegalArgumentException("replace on sort Selector invoked with term of incorrect sort, String expected");

            return (katja.spec.Selector) Katja.unique(new katja.spec.Selector.Impl(
                pos == 0 ? (java.lang.String) term : _name
            ));
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.SelectorOpt.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseSelector(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Katja.Selector");
            builder.append("( ");
            builder.append("\"").append(_name.toString()).append("\"");
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("Selector");
            builder.append("( ");
            builder.append("\"").append(_name.toString()).append("\"");
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "Selector";
        }
    }
}

