package katja.spec;

import katja.common.*;
import java.io.IOException;

public interface TupleParam extends KatjaTuple {

    //----- methods of TupleParam -----

    public katja.spec.SortId sortId();
    public katja.spec.TupleParam replaceSortId(katja.spec.SortId sortId);
    public katja.spec.SelectorOpt selector();
    public katja.spec.TupleParam replaceSelector(katja.spec.SelectorOpt selector);
    public java.lang.Boolean isOpt();
    public katja.spec.TupleParam replaceIsOpt(java.lang.Boolean isOpt);
    public Object get(int i);
    public int size();
    public katja.spec.TupleParam replace(int pos, Object term);

    //----- nested classes of TupleParam -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitSortId(katja.spec.SortId term) throws E;
        public void visitSelectorOpt(katja.spec.SelectorOpt term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitSelector(katja.spec.Selector term) throws E;
        public void visitEmpty(katja.spec.Empty term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitTupleParam(katja.spec.TupleParam term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.TupleParam.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final katja.spec.SelectorOpt.Switch<Object, E> variantVisit$SelectorOpt = new katja.spec.SelectorOpt.Switch<Object, E>() { public final Object CaseSelector(katja.spec.Selector term) throws E { visitSelector(term); return null; } public final Object CaseEmpty(katja.spec.Empty term) throws E { visitEmpty(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitSelectorOpt(katja.spec.SelectorOpt term) throws E {
            term.Switch(variantVisit$SelectorOpt);
        }
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.TupleParam {

        //----- attributes of Impl -----

        private katja.spec.SortId _sortId = null;
        private katja.spec.SelectorOpt _selector = null;
        private java.lang.Boolean _isOpt = null;

        //----- methods of Impl -----

        public katja.spec.SortId sortId() {
            return _sortId;
        }

        public katja.spec.TupleParam replaceSortId(katja.spec.SortId sortId) {
            return replace(0, sortId);
        }

        public katja.spec.SelectorOpt selector() {
            return _selector;
        }

        public katja.spec.TupleParam replaceSelector(katja.spec.SelectorOpt selector) {
            return replace(1, selector);
        }

        public java.lang.Boolean isOpt() {
            return _isOpt;
        }

        public katja.spec.TupleParam replaceIsOpt(java.lang.Boolean isOpt) {
            return replace(2, isOpt);
        }

        Impl(katja.spec.SortId sortId, katja.spec.SelectorOpt selector, java.lang.Boolean isOpt) {
            if(sortId == null)
                throw new IllegalArgumentException("constructor of sort TupleParam invoked with null parameter sortId");
            if(selector == null)
                throw new IllegalArgumentException("constructor of sort TupleParam invoked with null parameter selector");
            if(isOpt == null)
                throw new IllegalArgumentException("constructor of sort TupleParam invoked with null parameter isOpt");

            this._sortId = sortId;
            this._selector = selector;
            this._isOpt = isOpt;
        }

        public Object get(int i) {
            int ith = i;

            if(ith < 0) ith += 3;

            switch(ith) {
                case 0: return _sortId;
                case 1: return _selector;
                case 2: return _isOpt;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort TupleParam invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort TupleParam invoked with too large parameter "+i+", sort has only 3 components");
                    }
            }
        }

        public int size() {
            return 3;
        }

        public katja.spec.TupleParam replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort TupleParam invoked with negative parameter "+pos);
            if(pos >= 3)
                throw new IllegalArgumentException("replace on sort TupleParam invoked with too large parameter "+pos+", sort has only 3 components");

            if(pos == 0 && !(term instanceof katja.spec.SortId))
                throw new IllegalArgumentException("replace on sort TupleParam invoked with term of incorrect sort, SortId expected");
            if(pos == 1 && !(term instanceof katja.spec.SelectorOpt))
                throw new IllegalArgumentException("replace on sort TupleParam invoked with term of incorrect sort, SelectorOpt expected");
            if(pos == 2 && !(term instanceof java.lang.Boolean))
                throw new IllegalArgumentException("replace on sort TupleParam invoked with term of incorrect sort, Boolean expected");

            return (katja.spec.TupleParam) Katja.unique(new katja.spec.TupleParam.Impl(
                pos == 0 ? (katja.spec.SortId) term : _sortId,
                pos == 1 ? (katja.spec.SelectorOpt) term : _selector,
                pos == 2 ? (java.lang.Boolean) term : _isOpt
            ));
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Katja.TupleParam");
            builder.append("( ");
            _sortId.toJavaCode(builder);
            builder.append(", ");
            _selector.toJavaCode(builder);
            builder.append(", ");
            builder.append(_isOpt.toString());
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("TupleParam");
            builder.append("( ");
            _sortId.toString(builder);
            builder.append(", ");
            _selector.toString(builder);
            builder.append(", ");
            builder.append(_isOpt.toString());
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "TupleParam";
        }
    }
}

