package katja.spec;

import katja.common.*;
import java.io.IOException;

public interface TupleProd extends katja.spec.Production, KatjaTuple {

    //----- methods of TupleProd -----

    public katja.spec.SortId sortId();
    public katja.spec.TupleProd replaceSortId(katja.spec.SortId sortId);
    public katja.spec.TupleParamList params();
    public katja.spec.TupleProd replaceParams(katja.spec.TupleParamList params);
    public KatjaTerm get(int i);
    public int size();
    public katja.spec.TupleProd replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(katja.spec.Production.Switch<CT, E> switchClass) throws E;
    public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E;
    public <CT, E extends Throwable> CT Switch(katja.spec.VariantParam.Switch<CT, E> switchClass) throws E;

    //----- nested classes of TupleProd -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitSortId(katja.spec.SortId term) throws E;
        public void visitTupleParamList(katja.spec.TupleParamList term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitTupleParam(katja.spec.TupleParam term) throws E;
        public void visitSelectorOpt(katja.spec.SelectorOpt term) throws E;
        public void visitSelector(katja.spec.Selector term) throws E;
        public void visitEmpty(katja.spec.Empty term) throws E;
        public void visitTupleProd(katja.spec.TupleProd term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.TupleProd.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final katja.spec.SelectorOpt.Switch<Object, E> variantVisit$SelectorOpt = new katja.spec.SelectorOpt.Switch<Object, E>() { public final Object CaseSelector(katja.spec.Selector term) throws E { visitSelector(term); return null; } public final Object CaseEmpty(katja.spec.Empty term) throws E { visitEmpty(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitSelectorOpt(katja.spec.SelectorOpt term) throws E {
            term.Switch(variantVisit$SelectorOpt);
        }
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.TupleProd {

        //----- attributes of Impl -----

        private katja.spec.SortId _sortId = null;
        private katja.spec.TupleParamList _params = null;

        //----- methods of Impl -----

        public katja.spec.SortId sortId() {
            return _sortId;
        }

        public katja.spec.TupleProd replaceSortId(katja.spec.SortId sortId) {
            return replace(0, sortId);
        }

        public katja.spec.TupleParamList params() {
            return _params;
        }

        public katja.spec.TupleProd replaceParams(katja.spec.TupleParamList params) {
            return replace(1, params);
        }

        Impl(katja.spec.SortId sortId, katja.spec.TupleParamList params) {
            if(sortId == null)
                throw new IllegalArgumentException("constructor of sort TupleProd invoked with null parameter sortId");
            if(params == null)
                throw new IllegalArgumentException("constructor of sort TupleProd invoked with null parameter params");

            this._sortId = sortId;
            this._params = params;
        }

        public KatjaTerm get(int i) {
            int ith = i;

            if(ith < 0) ith += 2;

            switch(ith) {
                case 0: return _sortId;
                case 1: return _params;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort TupleProd invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort TupleProd invoked with too large parameter "+i+", sort has only 2 components");
                    }
            }
        }

        public int size() {
            return 2;
        }

        public katja.spec.TupleProd replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort TupleProd invoked with negative parameter "+pos);
            if(pos >= 2)
                throw new IllegalArgumentException("replace on sort TupleProd invoked with too large parameter "+pos+", sort has only 2 components");

            if(pos == 0 && !(term instanceof katja.spec.SortId))
                throw new IllegalArgumentException("replace on sort TupleProd invoked with term of incorrect sort, SortId expected");
            if(pos == 1 && !(term instanceof katja.spec.TupleParamList))
                throw new IllegalArgumentException("replace on sort TupleProd invoked with term of incorrect sort, TupleParamList expected");

            return (katja.spec.TupleProd) Katja.unique(new katja.spec.TupleProd.Impl(
                pos == 0 ? (katja.spec.SortId) term : _sortId,
                pos == 1 ? (katja.spec.TupleParamList) term : _params
            ));
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.Production.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseTupleProd(this);
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.SortSpec.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseTupleProd(this);
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.VariantParam.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseTupleProd(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Katja.TupleProd");
            builder.append("( ");
            _sortId.toJavaCode(builder);
            builder.append(", ");
            _params.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("TupleProd");
            builder.append("( ");
            _sortId.toString(builder);
            builder.append(", ");
            _params.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "TupleProd";
        }
    }
}

