package katja.spec.isabelle;

import katja.common.*;
import java.io.IOException;

public interface TupleSelector extends katja.spec.isabelle.PositionSelector, KatjaTuple {

    //----- methods of TupleSelector -----

    public katja.spec.descriptors.SelectorDescriptor selector();
    public katja.spec.isabelle.TupleSelector replaceSelector(katja.spec.descriptors.SelectorDescriptor selector);
    public katja.spec.descriptors.SelectorDescriptor get(int i);
    public int size();
    public katja.spec.isabelle.TupleSelector replace(int pos, Object term);
    public <CT, E extends Throwable> CT Switch(katja.spec.isabelle.PositionSelector.Switch<CT, E> switchClass) throws E;

    //----- nested classes of TupleSelector -----

    static interface VisitorType<E extends Throwable> {

        //----- methods of VisitorType<E extends Throwable> -----

        public void visitSelectorDescriptor(katja.spec.descriptors.SelectorDescriptor term) throws E;
        public void visitSortDescriptor(katja.spec.descriptors.SortDescriptor term) throws E;
        public void visitInteger(java.lang.Integer term) throws E;
        public void visitString(java.lang.String term) throws E;
        public void visitTermSortDescriptor(katja.spec.descriptors.TermSortDescriptor term) throws E;
        public void visitPosSortDescriptor(katja.spec.descriptors.PosSortDescriptor term) throws E;
        public void visitTupleTermSortDescriptor(katja.spec.descriptors.TupleTermSortDescriptor term) throws E;
        public void visitListTermSortDescriptor(katja.spec.descriptors.ListTermSortDescriptor term) throws E;
        public void visitVariantTermSortDescriptor(katja.spec.descriptors.VariantTermSortDescriptor term) throws E;
        public void visitExternTermSortDescriptor(katja.spec.descriptors.ExternTermSortDescriptor term) throws E;
        public void visitBoolean(java.lang.Boolean term) throws E;
        public void visitPosStructDescriptor(katja.spec.descriptors.PosStructDescriptor term) throws E;
        public void visitTupleSelector(katja.spec.isabelle.TupleSelector term) throws E;
    }

    public static abstract class Visitor<E extends Throwable> implements katja.spec.isabelle.TupleSelector.VisitorType<E> {

        //----- attributes of Visitor<E extends Throwable> -----

        private final katja.spec.descriptors.SortDescriptor.Switch<Object, E> variantVisit$SortDescriptor = new katja.spec.descriptors.SortDescriptor.Switch<Object, E>() { public final Object CasePosSortDescriptor(katja.spec.descriptors.PosSortDescriptor term) throws E { visitPosSortDescriptor(term); return null; } public final Object CaseTupleTermSortDescriptor(katja.spec.descriptors.TupleTermSortDescriptor term) throws E { visitTupleTermSortDescriptor(term); return null; } public final Object CaseListTermSortDescriptor(katja.spec.descriptors.ListTermSortDescriptor term) throws E { visitListTermSortDescriptor(term); return null; } public final Object CaseVariantTermSortDescriptor(katja.spec.descriptors.VariantTermSortDescriptor term) throws E { visitVariantTermSortDescriptor(term); return null; } public final Object CaseExternTermSortDescriptor(katja.spec.descriptors.ExternTermSortDescriptor term) throws E { visitExternTermSortDescriptor(term); return null; } };

        //----- methods of Visitor<E extends Throwable> -----

        public final void visitSortDescriptor(katja.spec.descriptors.SortDescriptor term) throws E {
            term.Switch(variantVisit$SortDescriptor);
        }

        public final void visitTermSortDescriptor(katja.spec.descriptors.TermSortDescriptor term) throws E {
            term.Switch(variantVisit$SortDescriptor);
        }
    }

    static class Impl extends KatjaTupleImpl implements katja.spec.isabelle.TupleSelector {

        //----- attributes of Impl -----

        private katja.spec.descriptors.SelectorDescriptor _selector = null;

        //----- methods of Impl -----

        public katja.spec.descriptors.SelectorDescriptor selector() {
            return _selector;
        }

        public katja.spec.isabelle.TupleSelector replaceSelector(katja.spec.descriptors.SelectorDescriptor selector) {
            return replace(0, selector);
        }

        Impl(katja.spec.descriptors.SelectorDescriptor selector) {
            if(selector == null)
                throw new IllegalArgumentException("constructor of sort TupleSelector invoked with null parameter selector");

            this._selector = selector;
        }

        public katja.spec.descriptors.SelectorDescriptor get(int i) {
            int ith = i;

            if(ith < 0) ith += 1;

            switch(ith) {
                case 0: return _selector;
                default:
                    if(ith < 0) {
                        throw new IllegalArgumentException("get on sort TupleSelector invoked with negative parameter "+i);
                    } else {
                        throw new IllegalArgumentException("get on sort TupleSelector invoked with too large parameter "+i+", sort has only 1 components");
                    }
            }
        }

        public int size() {
            return 1;
        }

        public katja.spec.isabelle.TupleSelector replace(int pos, Object term) {
            if(pos < 0)
                throw new IllegalArgumentException("replace on sort TupleSelector invoked with negative parameter "+pos);
            if(pos >= 1)
                throw new IllegalArgumentException("replace on sort TupleSelector invoked with too large parameter "+pos+", sort has only 1 components");

            if(pos == 0 && !(term instanceof katja.spec.descriptors.SelectorDescriptor))
                throw new IllegalArgumentException("replace on sort TupleSelector invoked with term of incorrect sort, SelectorDescriptor expected");

            return (katja.spec.isabelle.TupleSelector) Isabelle.unique(new katja.spec.isabelle.TupleSelector.Impl(
                pos == 0 ? (katja.spec.descriptors.SelectorDescriptor) term : _selector
            ));
        }

        public <CT, E extends Throwable> CT Switch(katja.spec.isabelle.PositionSelector.Switch<CT, E> switchClass) throws E {
            return switchClass.CaseTupleSelector(this);
        }

        public Appendable toJavaCode(Appendable builder) throws IOException {
            builder.append("Isabelle.TupleSelector");
            builder.append("( ");
            _selector.toJavaCode(builder);
            builder.append(" )");

            return builder;
        }

        public Appendable toString(Appendable builder) throws IOException {
            builder.append("TupleSelector");
            builder.append("( ");
            _selector.toString(builder);
            builder.append(" )");

            return builder;
        }

        public final String sortName() {
            return "TupleSelector";
        }
    }
}

